#import "lib/admonitions.typ": todo
#import "@preview/lovelace:0.2.0": *
#import "lib/modern-unito-thesis-stripped.typ": template
#import "@preview/colorful-boxes:1.1.0": stickybox
#import "@preview/wordometer:0.1.0": total-words, word-count
#import "@preview/drafting:0.2.0": set-page-properties, set-margin-note-defaults

// change to true for final export
#let production = "PRODUCTION" in sys.inputs;

#set document(title: "Robot Receptionist", author: "Pawel Makles")
#set page(paper: "a4", margin: (right: 3cm, left: 3.5cm, top: 3.5cm, bottom: 3.5cm), footer: context[
  #if production [
    #text(gray)[6CCS3PRJ Report Final Appendix Copy]
  ] else [
    #text(red)[*DRAFT COPY #datetime.today().display()*]
  ]
  #h(1fr)
  #counter(page).display("1/1", both: true)
])

// configure lovelace
#show: setup-lovelace
// configure drafting:
#set-page-properties()
// configure word count:
#show: word-count.with(exclude: strike)

// configure codly
#import "@preview/codly:0.2.0": *
#show: codly-init.with()

#let icon(codepoint) = {
  box(height: 0.8em, baseline: 0.05em, image(codepoint))
  h(0.1em)
}

#codly(languages: (
  rust: (name: "Rust", icon: icon("images/icons/brand-rust.svg"), color: rgb("#CE412B")),
  yaml: (name: "YAML", icon: icon("images/icons/brand-yaml.svg"), color: rgb("#35A235")),
  python: (name: "Python", icon: icon("images/icons/brand-python.svg"), color: rgb("#3572A5")),
), zebra-color: white)

#show figure.where(kind: table): set figure.caption(position: top)
#show figure.where(kind: "data"): set figure.caption(position: top)

// just some fun, enable sticky notes:
#let default-rect(stroke: none, fill: none, width: 0pt, content) = {
  set text(0.9em)
  stickybox(rotation: 20deg, width: width, content)
}
#set-margin-note-defaults(rect: default-rect, stroke: rgb(0, 0, 0, 20), side: right)

#let declaration-of-originality = []

#let acknowledgments = []

#let abstract = []

#show: template.with(
  title: "Robot Receptionist",
  academic-year: [2023/2024],
  subtitle: "6CCS3PRJ Project Report",
  candidate: (name: "Pawel Makles", number: "K21002534"),
  supervisor: ("Dr. Matteo Leonetti"),
  co-supervisor: none,
  affiliation: (university: "King's College London", school: "Department of Informatics", degree: "MSci Computer Science"),
  lang: "en",
  logo: image("images/template/logo.png", width: 40%),
  bibliography: bibliography("references.bib", style: "ieee"),
  declaration-of-originality: declaration-of-originality,
  acknowledgments: acknowledgments,
  abstract: abstract,
  keywords: none,
)

#set heading(numbering: "A.1.1")
#counter(heading).update(0)

= User Manual
#include "chapters/appendix/user-manual.typ"

#pagebreak(weak: true)
= Receptionist Task Rules
#include "chapters/appendix/receptionist_task.typ"

#pagebreak(weak: true)
= Raw Experiment Test Data
#include "chapters/appendix/experiment-results.typ"
