#import "@preview/wrap-it:0.1.0": wrap-content
#import "@preview/drafting:0.2.0": margin-note
#import "../lib/admonitions.typ": todo, disclaimer
#import "../lib/diagrams.typ": fletcher, edge, node, blob, ros-node

== TIAGo

For this project, I will be using the PAL Robotics TIAGo "mobile-manipulator" robot @tiago to implement the task, it is
a generic robot platform that serves as an excellent basis for building service robotics applications. The robot
features: A *LiDAR* which is a special device that uses lasers to determine the distance of objects around the robot,
this is necessary for the robot to be able to determine where it is in the world and subsequently navigate through it.
It has a *differential drive* base which means it can turn in place and move both forwards and backward, this means the
robot can follow more natural and concise movements akin to humans.

An *RGBD camera* is used to map out the environment around the robot, while also being able to correlate this with
information gathered from its 2D view. A *mechanical arm* is present which can be used for object manipulation, however,
this is not necessary for this task. The *onboard computer* is used to run and operate the robot, it is effectively
immutable to us and we need not modify it. Instead, we connect to it over the network and run our logic on another
machine.

== ROS Overview

The *Robot Operating System* is a "meta-operating system" that provides a common platform for us to interact with many
supported robots. We will be using ROS Noetic Nimjemys which is the LTS release of ROS1 based on Ubuntu 20.04 LTS, which
is compatible with our robot TIAGo.

A *node* is a "process that performs a computation" @ros_node, it represents an independent program that runs somewhere
on the robot's network that can provide or consume information, but crucially, nodes do not need to care about how data
is sent to/from them as this is all handled by the respective ROS client libraries for each language. In our case, we
will be using rospy @rospy to create our nodes.

#figure(caption: "Nodes on the network can provide different functionality", fletcher.diagram(
  node-stroke: 0.5pt,
  edge-stroke: 0.7pt,
  ros-node((0, 0), "/node1", [*topic* /status], tint: purple),
  ros-node((1, -0.5), "/node2", [*service* /doSomething], tint: green),
  ros-node((1, 0.5), "/node3", tint: red),
  edge((0, 0.15), (0.35, 0.15), (0.4, 0.5), (1, 0.5), "->", "subscribed"),
  edge((1, 0.5), (1.8, 0.5), (1.8, -0.35), (1, -0.35), "-->", "service call"),
))

Before we can think about sending data, we need to look at message types. ROS implements a common data format for
efficient transportation across the network, we define `.msg` files in our project which define key-value entries. The
value can be one of many types such as boolean, differently-sized and signed integers, differently-sized floats,
strings, and time/duration.

#figure(kind: "data", supplement: [Data Structure], caption: "Example Message Definition", table(
  columns: (auto),
  inset: 5pt,
  align: horizon + left,
  table.header([*MyMessage.msg*]),
  [\- header: *std_msgs/Header* #linebreak() \- pose: *geometry_msgs/Pose* #linebreak() \- count: *int*],
)) <data-type-message>

#disclaimer[
  ROS messages described throughout the paper are not entirely spec-compliant as they have been simplified for brevity.
  The implementation section will later discuss the challenges of applying complex data structures to ROS message types.
]

When it comes to this project, two packages provide especially important messages, `std_msgs` and `geometry_msgs`.
Specifically the message definitions for: *std_msgs/Header* @rosmsg_std_msgs_Header which includes the current time, a
frame of reference, and a unique sequence ID. The *geometry_msgs/Point* @rosmsg_geometry_msgs_point which is a 3D
coordinate. And the *geometry_msgs/Pose* @rosmsg_geometry_msgs_pose which includes a point defined as before and a
quaternion for rotation.

#figure(
  kind: "data",
  supplement: [Data Structure],
  caption: "Important Message Definitions",
  table(columns: (auto, auto, auto, auto), inset: 5pt, stroke: none, table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*Header*]),
    [\- seq: *uint32* #linebreak() \- stamp: *time* #linebreak() \- frame_id: *str*],
  ), table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*Point*]),
    [\- x: *float* #linebreak() \- y: *float* #linebreak() \- z: *float*],
  ), table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*Quaternion*]),
    [\- x: *float* #linebreak() \- y: *float* #linebreak() \- z: *float* #linebreak() \- w: *float*],
  ), table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*Pose*]),
    [\- point: *Point* #linebreak() \- orientation: *Quaternion*],
  )),
) <data-type-message>

When using ROS there are 3 primary ways to send data between nodes: A *topic* is a channel through which we can send
messages over the ROS network @ros_topic, anybody may subscribe or publish to any topic they wish but the message type
must be negotiated ahead of time. A *service* is defined by two message types defined in a `.srv` file (see
@data-type-service), the request and the response, a node may send a request to a node that provides a given service and
will receive a response with the result @ros_service. Lastly, we can also use the *parameter server* to persistently
store some chunks of data @ros_param, in essence, it is a dictionary that lives on the network that we can read and
write from at any time, it supports a multitude of data types and we can use this to store configurations that lives
outside of our program and persists between invocations.

#align(
  center,
  grid(
    columns: (auto, auto),
    inset: 5pt,
    [#figure(kind: "data", supplement: [Data Structure], caption: "Example Service Definition", table(
        columns: (auto),
        inset: 5pt,
        align: horizon + left,
        table.header([*MyService.srv*]),
        [\- arg1: *string* #linebreak() \- arg2: *int*],
        [\- result: *string*],
      )) <data-type-service>],
    [#figure(kind: "data", supplement: [Data Structure], caption: "Example Parameter Server Data", fletcher.diagram(
        node-stroke: 0pt,
        edge-stroke: 0.3pt,
        node((0, 0.38), "/parent_key"),
        edge((0, 0.38), (1, -0.05), "->"),
        node((1, 0), [/param1: *string*]),
        node((1, 0.4), [/count: *int*]),
        edge((0, 0.4), (1, 0.4), "->"),
      )) <data-parameter-server>],
  ),
)

Sometimes, we want to be able to cancel or monitor the status of a given task, we could build this infrastructure
ourselves or use an existing well-integrated framework called *actionlib* @ros_actionlib. This library provides a way to
create action servers that accept given requests, regularly give feedback on the request, and allow requests to be
preempted.

#figure(kind: "data", supplement: [Data Structure], caption: "Example Action Definition", table(
  columns: (auto),
  inset: 5pt,
  align: horizon + left,
  table.header([*MyAction.action*]),
  [\- arg1: *string* #linebreak() \- arg2: *int*],
  [\- progress: *float*],
  [\- result: *string*],
)) <data-type-action>

*tf2* is a library that lets us "keep track of coordinate frames over time" @ros_tf2 by keeping a buffer of changes in
relationships between different coordinate frames in memory. We can use it to transform points between different
coordinate frames such as the robot's camera, the robot's base, or the map. This allows us to, for example, find out
where an object relative to the robot's camera is positioned absolutely within the map. tf can also perform
transformations at given points in time assuming that information is still in the buffer.

*SMACH* @smach is a powerful and extensible library for building state machines, it has built-in support for data
passing between states and preempting running states, and also provides us with many useful concepts out of the box such
as concurrency and iteration. It is also well integrated into the ROS ecosystem through the use of the *smach_ros*
library @smach_ros, which gives us additional helper classes that wrap around ROS concepts such as services and actions.

#figure(fletcher.diagram(
  node-stroke: 0.5pt,
  edge-stroke: 0.7pt,
  blob((0, 0), "Start", tint: gray),
  edge("->"),
  blob((0, 0.8), "Perform computation"),
  edge("->", "{userdata}"),
  blob((0, 1.6), "Loop", tint: yellow),
  edge("->", "i = 0..5"),
  blob((1, 1.6), "Another state machine", tint: red),
  edge("->", "{output}"),
  blob((1, 2.4), "Process output"),
  edge((1, 2.4), (0.5, 2.4), (0, 1.6), "->"),
  edge((0, 1.6), (0, 2.4), "-->", "exhausted"),
  blob((0, 2.4), "End", tint: gray),
  edge((0, 0.8), (-1, 0.8), (-1, 1.6), "->"),
  blob((-1, 1.6), "Concurrent operation"),
  edge((-1.4, 1.6), (-1.4, 2), (-1.8, 2), (-1.8, 1.6), (-1, 1.6), "->"),
  edge((-1, 1.6), (-1, 2.4), (0, 2.4), "-->", "preempted"),
), caption: "Example state machine") <smach-example>

#wrap-content(
  figure(caption: "RViz", image("../images/screenshots/Screenshot from 2024-03-22 13-31-24.png", width: 120pt)),
  [We also want a way to visualise data coming from the robot, for this, we use rviz @rizz which provides us with the
    ability to see depth and map information in 3D space as well as view individual topics and simultaneously other 2D
    information such as the camera feed.],
  align: right,
)

#wrap-content(
  figure(caption: "Gazebo", image("../images/screenshots/Screenshot from 2024-03-22 13-31-14.png", width: 120pt)),
  [For testing we also need to be able to simulate a robot in software, for this, we use Gazebo @gazebo. The TIAGo software
    package already provides us with the necessary resources required to use Gazebo to simulate the TIAGo robot.],
  align: right,
)

#wrap-content(
  figure(caption: "Foxglove", image("../images/screenshots/Screenshot 2024-04-03 at 20.07.56.png", width: 160pt)),
  [Sometimes we may want to record and look back on previous experiments or invocations of our code, or otherwise, we might
    find it tricky or time-consuming to debug issues with our code on either the real robot or even in simulation. For this,
    we can record messages sent to topics into special files called *bags* using the _rosbag_ utility @rosbag. We can then
    play these back over a ROS network to view and capture the data through things such as rviz. However, handling rosbags
    in this way can also prove to be tedious and time-consuming, so instead, we can use a tool called Foxglove @foxglove
    which allows us to load individual bags and inspect all the data using a timeline view.],
)

One last note on ROS: In order to simplify starting and managing different configurations for lots of nodes, we can use
roslaunch files @roslaunch which are XML files that define the nodes and arguments that we want to load. Arguments to
nodes can be passed via command line arguments or through the parameter server using private parameters, once defined
they may be accessed through a special parameter namespace \~, e.g. the parameter `hello` is mapped to `~hello`.

=== TIAGo Actions

TIAGo provides several useful action servers that will be used throughout the project. The important ones to remember
are the navigation stack `/move_base`, a playback server for predefined motions `/play_motion`, and a text-to-speech
server that uses the onboard speakers `/tts`.

#figure(
  kind: "data",
  supplement: [Data Structure],
  caption: "Key Action Definitions for TIAGo",
  table(columns: (auto, auto, auto), inset: 5pt, stroke: none, table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*MoveBaseGoal.msg*]),
    [\- header: *std/Header* #linebreak() \- pose: *geom/Pose*],
  ), table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*PlayMotionGoal.msg*]),
    [\- motion_name: *str* #linebreak() \- skip_planning: *bool*],
  ), table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*TtsGoal.msg*]),
    [\- rawtext.text: *str* #linebreak() \- rawtext.lang_id = "*en_GB*"],
  )),
) <data-tiago-actions>

The play motion server will read motion information from the parameter server using the interpolated key
`/play_motion/{motion_name}`, so we can load our motions into parameters to later play them back. In the case of all of
these actions, they will always run to completion before succeeding so there is no risk of, for example, starting to
listen for speech while the robot is still saying something.

#pagebreak(weak: true)
== Prior Art

This project dips into many adjacent fields such as computer vision and speech recognition, this section intends to
introduce the many tools and prior work done by people that will be discussed later.

*YOLO* (You-Only-Look-Once) is an ongoing research project spanning dozens of individuals into one-shot computer object
recognition. Currently, the latest stable software release is YOLOv8 by ultralytics @Jocher_YOLO_by_Ultralytics_2023,
with a new YOLOv9 paper @wang2024yolov9 on the horizon and being implemented however it is not stable enough for use
yet.

*PyTorch* is a deep machine learning framework that we can use to train custom computer vision models to better suit our
needs. *DeepFace* is a Python framework @deepface-framework that provides us access to state-of-the-art facial feature
extraction models that allow us to perform facial reidentification. *BodyPix* is a machine learning model that allows us
to segment body parts from pictures of people @bodypix @bodypix-python. *Weaviate* @weaviate is an open-source vector
database.

*Whisper* is a speech recognition model built by OpenAI @radford2022whisper that offers outstanding performance over
many different languages. *RASA* is a complementary framework for creating and training models to recognise the intent
in dialog @bocklisch2017rasa.

*LASR* (Learning Autonomous Service Robots) is a team of students from King's College London "competing in service
robotics challenges at major competitions" @lasr. We publish reusable software components that will also be integrated
into this project, in particular, *lasr-skills* @lasr-skills which offers many commonly-used states for SMACH, intended
for use with TIAGo.