#import "../../lib/admonitions.typ": todo
#pagebreak(weak: true)
== Approaches to face reidentification

Face reidentification consists of two problems, how do we find a face in an image and how do we match faces to each
other?

First, we focus on face recognition which is essentially a solved problem, work on face recognition as far back as 2014
was already achieving results with accuracies of "97.35% on the Labeled Faces in the Wild" dataset @6909616. Labeled
faces in the wild @huang2008labeled being a popular benchmark used to test these models. The current consensus is that
the human accuracy on this dataset is about 97.53% @10127799. VGG-Face @vgg_face @Parkhi15, a model developed by a group
at the University of Oxford, achieved 98.78% @10127799 accuracy which surpasses human ability.

Two other notable models include SFace @Sface_Boutros, which is a model built to address privacy concerns with existing
face training datasets; breaking from the status quo, SFace is trained on synthetic face images and performs exceedingly
well with 99.60% accuracy @10127799. Facenet512 @Schroff_2015, which is the best-performing model seen so far with
99.63% accuracy @10127799.

Next, we need to tackle matching previously learned faces to newly gathered face images. All of these models produce
mathematical representations, or vectors, of given faces, so we just need to figure out how we can store and look them
up. The DeepFace framework @deepface-framework provides an end-to-end solution as a Python library that provides access
to most of the facial recognition models mentioned above but also allows us to create a local database of face images,
from which the library automatically builds and manages a cache of the vector output from each model. The library then
looks up the closest vector to match faces.

However, the scalability of using a local directory of face images with a cache is not great, so we can instead opt to
use a purpose-built vector database. Weaviate @weaviate is an "open-source vector database" that can let us store and
lookup vectors as well as any arbitrary associated data that we care about, in the context of this project, this would
be a unique identifier for a guest such as an index or a name.

We also see that there is an alternative approach to solving this problem by implementing a generic 'image search
engine' @fireship_image_search using the previously mentioned vector database. Weaviate features built-in vectorisers
such as text-to-vec, image-to-vec, and CLIP. One could use the resnet50 image-to-vec vectoriser in order to convert face
images into vectors and then look them up after the fact. (though we would first have to align to the face, which we can
do using the body segmentation mentioned earlier or using a YOLO model trained on faces @qi2022yolo5face)