#import "../../lib/admonitions.typ": todo
== Reliable and robust conversations

To be able to hold a conversation with a user, we must implement three key features: text-to-speech, speech-to-text, and
dialog intent recognition. Text-to-speech is already implemented for us on the TIAGo platform, so we just need to find
adequate solutions for speech and intent recognition.

=== Recognising human speech

Mozilla DeepSpeech is an "open source embedded speech-to-text engine" that can run in real-time on a variety of high and
low-power devices @MozillaDeepSpeech. It is based on a research paper by Baidu @hannun2014deep that presents an (at the
time) "state-of-the-art speech recognition model" that achieves a 16% error rate @hannun2014deep on the Switchboard
Hub5'00 dataset.

Vosk @Vosk is another offline speech recognition model that is especially notable because it has first-class support for
ROS through their ros_vosk library @ros_vosk. A meta-review by AlphaCephei @Shmyrev_2022 (the organisation behind vosk)
describes the relative performance of vosk to other models including two trained on the GigaSpeech dataset
@GigaSpeech2021, NVidia Transducer @stt_en_conformer_transducer_xlarge, and the various OpenAI Whisper
@radford2022whisper models (tiny.en, small.en, large.en).

The meta-review concludes that Vosk's accuracy is "outdated, needs to improve as soon as possible" and that relative to
Whisper, "other models are not that bad too like GigaSpeech models [..], if you look for best accuracy they are about
the same" @Shmyrev_2022. Whisper itself can perform speech recognition in real-time even with constrained resources and
is quite accurate in noisy environments so it is an appropriate choice for this component of the project.

// There are also many commercial/cloud offerings from Google, IBM, OpenAI, Microsoft Azure, AssemblyAI, Rev, Speechmatics, and Deepgram to name a few.

// https://cloud.google.com/speech-to-text/
// https://www.ibm.com/products/speech-to-text
// https://platform.openai.com/docs/guides/speech-to-text
// https://azure.microsoft.com/en-us/products/ai-services/speech-to-text
// https://www.assemblyai.com/products/speech-to-text
// => https://www.assemblyai.com/research/universal-1
// https://www.rev.ai/
// https://www.speechmatics.com/product/transcription
// https://deepgram.com/learn/nova-2-speech-to-text-api

#pagebreak(weak: true)
=== Recognising speech intent

Dialogflow is "a natural language understanding platform [.. for] conversational interfaces" @Sabharwal2020. An
evaluation by Muhammad et al. found almost all the agents they designed using DialogFlow had an accuracy rate of 100%
@9231659. However, the use of DialogFlow requires us to interact with a cloud-hosted API which comes with additional
charges @Dialogflow. While the paid API is not necessarily a deal-breaker as Google provides enough free credit to
develop the solution, the always-online/latency as a result of this architecture makes it unsuitable for this project.

RASA @bocklisch2017rasa is a "conversational AI framework" that allows you to train a neural network to understand and
interpret conversation transcripts. It would prove very easy to integrate into the project as they provide a Python
library we can hook into @pypi_rasa and a straightforward CLI interface for building and training new models @rasa_cli.