#import "../../lib/admonitions.typ": todo
== One-shot person and object recognition

YOLO (You Only Look Once) is a "widely used algorithm" for one-shot object detection @JIANG20221066. It was first
unveiled in 2015 and presented a new approach to object detection. The 'object detection' problem is reframed as a 'regression
problem' where "a single neural network predicts bounding boxes and class probabilities directly from full images"
@8029130.

#figure(
  caption: [Core concept of the YOLO algorithm (from @8029130)],
  image("../../images/screenshots/Screenshot_20240408_193238.png", width: 65%),
)

Over the years, there have been many successor algorithms including YOLOv2-5 @JIANG20221066, each attempting to optimise
performance by simplifying the network or extending it to improve accuracy. More recent work such as YOLOv6
@li2022yolov6 and YOLOv7 @Wang_2023_CVPR achieve real-time performance with reasonable accuracy.

Building on all of this prior work, YOLOv8 @Jocher_YOLO_by_Ultralytics_2023 is the current state-of-the-art model that
provides image classification, object detection, object segmentation, object tracking, and human pose inference. The
project claims much lower latency, with greater precision, using much smaller parameter models. To understand their
evaluation, we need to look at the metrics used.

The IoU measures overlap between two given boundaries (usually the real object boundary and our predicted boundary). It
is given by: $ "IoU" = frac("area of overlap", "area of intersection") $

So the IoU threshold is the minimum IoU between two given boundaries for a bounding box or mask to be considered a true
positive, otherwise it is discarded (false negative).

Given $N$ classes and $"AP"_i$ is the average precision of a class $i$ @coco_eval, the Mean Average Precision is given
by: $ "mAP" = frac(1, N) sum^n_{i=1} "AP"_i $

#figure(
  image("../../images/data/yolo-comparison-plots.png", width: 80%),
  caption: [Comparison of YOLO models (published by Ultralytics @yolo_comparison)],
)

These graphs show the Mean Average Precision (averaged over IoU thresholds of $.50$ and $.95$) of each model against the
parameters used and latency for inference. YOLOv8 dominates all other models at all parameter/latency tiers.

This project requires us to detect and segment chairs and people, so it is appropriate to use the pre-trained YOLO
models that are based on the COCO dataset @cocodataset. The dataset comprises 80 object categories which strikes a good
balance between having a dataset that can accurately determine one particular class but also has knowledge of many other
classes that it can then ignore and not falsely classify as a chair or person.