#import "../../lib/admonitions.typ": todo
#import "@preview/drafting:0.2.0": margin-note
#import "@preview/wrap-it:0.1.0": wrap-content
#import "../../lib/diagrams.typ": fletcher, edge, node, ros-node
== Nodes & Services

This project needs multiple auxiliary nodes that processing is offloaded to or otherwise that provide information
necessary for the completion of the task.

#let tiagoTopicMoveBase = -0.22;
#let tiagoTopicCamera = 0.28;
#let tiagoTopicCameraI = 0.03;

#figure(caption: "Interactions between nodes", fletcher.diagram(
  node-stroke: 0.5pt,
  edge-stroke: 0.7pt,
  ros-node((0, 0), "Task", tint: green),
  ros-node((1, 1), "Object Detection", [*service* /detect], tint: purple),
  ros-node((1, 2), "Body Segmentation", [*service* /segment], [*service* /face_features], tint: purple),
  ros-node((1, 3), "RASA", [*service* /parse], tint: purple),
  ros-node((2, 3), "Whisper", [*action* /transcribe], tint: purple),
  ros-node((2, 2), "DeepFace", [*service* /vectorise], tint: purple),
  ros-node((2, 1), "Weaviate", [*service* /insert], [*service* /query], tint: red),
  ros-node(
    (2, 0),
    "TIAGo",
    [*action* /move_base],
    [*topic* /pointcloud],
    [*action* /tts],
    [*topic* /camera/rgb_raw],
    tint: red,
  ),
  // task => nodes
  edge((0, 0), (2, tiagoTopicMoveBase), bend: 3deg),
  edge((0, 0), (2, tiagoTopicMoveBase + 0.24), bend: 3deg),
  edge((0, 0), (2, tiagoTopicMoveBase + 0.44), bend: 3deg),
  edge((0, 0), (0, 1.1), (1, 1.1)),
  edge((0, 0), (0, 2.12), (1, 2.12)),
  edge((0, 0), (0, 3.15), (1, 3.15)),
  // nodes => tiago
  edge((1, 1.1), (1.4, 1.1), (1.4, tiagoTopicCamera), (2, tiagoTopicCamera), "--"),
  edge(
    (1, 1.98),
    (1.42, 1.98),
    (1.42, tiagoTopicCamera + tiagoTopicCameraI),
    (2, tiagoTopicCamera + tiagoTopicCameraI),
    "--",
    "{img}",
  ),
  edge(
    (1, 2.2),
    (1.44, 2.2),
    (1.44, tiagoTopicCamera + tiagoTopicCameraI * 2),
    (2, tiagoTopicCamera + tiagoTopicCameraI * 2),
    "--",
  ),
  // body => deepface
  edge((1, 2.02), (1.5, 2.02), (1.5, 2.1), (2, 2.1), "-->"),
  // rasa => whisper
  edge((1, 3.15), (2, 3.15), "<--", "{speech}"),
  // deepface => weaviate
  edge((2, 2.1), (2.65, 2.1), (2.65, 1.2), (2, 1.2), "--"),
  edge((2, 2.14), (2.68, 2.14), (2.68, 1), (2, 1), "--", "{vec}"),
))

This diagram shows which services we must implement and their rough dependence on each other within the task. Dotted
lines indicate that this node relies on information that comes from another node, but not necessarily that the node
itself accesses it directly.

Navigation, a pointcloud (depth information mapped to 3D space), text-to-speech, and camera images are provided directly
by a node running on the robot indicated by *TIAGo*.

#pagebreak(weak: true)
The *Object Detection* node provides a service that accepts an image (we use a frame from the robot's camera), runs this
through a pre-trained COCO @cocodataset YOLOv8 @Jocher_YOLO_by_Ultralytics_2023 model and spits out detections in the
form of @data-yolo-detection. The actual service will be defined as follows.

#figure(
  kind: "data",
  supplement: [Data Structure],
  caption: "YOLO Detection Service",
  table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*YoloDetection.srv*]),
    [\- image: *sensor_msgs/Image* #linebreak() \- dataset: *string* #linebreak() \- confidence: *float* #linebreak() \- nms:
      *float*],
    [\- detected_objects: *YoloDetection[]*],
  ),
) <data-yolo-service>

The *dataset* value will be used to select the specific model that we want to use, the *confidence* value is the minimum
required confidence for a detection to not be pruned from the results, the *nms* (non-maximum suppression) is a guiding
value to remove overlapping detections.

#figure(
  kind: "data",
  supplement: [Data Structure],
  caption: "YOLO Detection Result",
  table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*YoloDetection.msg*]),
    [\- name: *string* #linebreak() \- confidence: *float* #linebreak() \- xywh: *int[]* #linebreak() \- xyseg: *int[]*],
  ),
) <data-yolo-detection>

Each detection provides the object found, the confidence that it is that object, the pixel-space coordinates of the
bounding box, and the pixel-space line segments that form the segmentation mask.

The *Body Segmentation* node will provide two services, one for segmenting an entire body and one for segmenting
features from a picture of a face. The service segmenting the entire body will accept an image and a list of requested
masks, it will then run the image through a chosen BodyPix @bodypix model (the two main options are using: resnet50 or
mobilenet50) with the given image and process the results by selecting the masks that were requested.

#figure(
  kind: "data",
  supplement: [Data Structure],
  caption: "Segment Body Service",
  table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*BodyPixDetection.srv*]),
    [\- image: *sensor_msgs/Image* #linebreak() \- dataset: *string* #linebreak() \- confidence: *float* #linebreak() \- masks:
      *string[][]*],
    [\- masks: *bool[][][]*],
  ),
) <data-bodypix-service>

The *dataset* and *confidence* values function in the same way as the object detection node, the *masks* value in the
request indicates a list of requested masks where we specify a list of body parts for each mask, and as such the *masks*
value in the response indicates a list of 2D arrays representing the segmented mask.

#pagebreak(weak: true)
The node will also provide a service that given an image of someone's head, will use a custom-trained PyTorch model to
segment their hair, face, glasses (if present), hat (if present). Using these masks, it will get the mean colour of the
hair, glasses, and hat. The service is defined as follows.

#figure(kind: "data", supplement: [Data Structure], caption: "Segment Face Service", table(
  columns: (auto),
  inset: 5pt,
  align: horizon + left,
  table.header([*FaceFeatures.srv*]),
  [\- image: *sensor_msgs/Image* #linebreak()],
  [\- features: *FeatureWithColour[]*],
)) <data-torch-service>

#figure(
  kind: "data",
  supplement: [Data Structure],
  caption: "Feature Information",
  grid(columns: (auto, auto), inset: 5pt, table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*FeatureWithColour.msg*]),
    [\- name: *string* #linebreak() \- colours: *ColourPrediction[]*],
  ), table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*ColourPrediction.msg*]),
    [\- colour: *string* #linebreak() \- distance: *float*],
  )),
) <data-feature>

The *RASA* node will be a simple wrapper around the RASA API, it is defined as follows.

#figure(kind: "data", supplement: [Data Structure], caption: "RASA Service", table(
  columns: (auto),
  inset: 5pt,
  align: horizon + left,
  table.header([*Rasa.srv*]),
  [\- text: *string*],
  [\- response: *string* (JSON-encoded string)],
)) <data-rasa-service>

The *Whisper* node will provide an action server for transcribing speech from a connected microphone, there will be no
arguments to the action. When invoked, it will start listening and capturing audio, transcribe it incrementally using
the Whisper speech-to-text model, and then return the final transcription when it believes the phrase is complete.

#figure(kind: "data", supplement: [Data Structure], caption: "Transcription Action", table(
  columns: (auto),
  inset: 5pt,
  align: horizon + left,
  table.header([*TranscribeSpeech.action*]),
  [],
  [\- transcribed_speech: *string*],
  [\- transcribed_speech: *string*],
)) <data-transcription-action>

The *DeepFace* node will provide a service that accepts an image with a face in it, vectorises it using a model provided
by the DeepFace framework @deepface-framework, and returns the vector accordingly. This service is defined as follows.

#figure(
  kind: "data",
  supplement: [Data Structure],
  caption: "DeepFace Service",
  grid(columns: (auto, auto), inset: 5pt, table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*DeepFace.srv*]),
    [\- image: *sensor_msgs/Image* #linebreak() \- model: *string*],
    [\- detected_faces: *Embedding[]*],
  ), table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*Embedding.msg*]),
    [\- vector: *float[]* #linebreak() \- confidence: *float* #linebreak() \- xywh: *int[]*],
  )),
) <data-deepface-service>

The *model*, *confidence*, and *xywh* values are like we've seen before, a *vector* is a series of floating point
numbers representing the face features.

The *Weaviate* node will be a thin wrapper around the Weaviate vector database, providing simple services that allow us
to insert a vector with one or more properties, and then query vectors.

Before we can use the database, we have to first initialise a collection for our vectors. We provide a service that can
do this and also handle the conditions where the collection exists, in which case we can either ignore it, error out, or
clear the collection.

#figure(kind: "data", supplement: [Data Structure], caption: "Weaviate Create Collection Service", table(
  columns: (auto),
  inset: 5pt,
  align: horizon + left,
  table.header([*CreateCollection.srv*]),
  [\- name: *string* #linebreak() \- skip_if_exists: *bool* #linebreak() \- clear_if_exists: *bool*],
  [],
)) <data-weaviate-create>

Now, we provide a service to insert vectors with any arbitrary properties.

#figure(
  kind: "data",
  supplement: [Data Structure],
  caption: "Weaviate Insert Vector Service",
  grid(columns: (auto, auto), inset: 5pt, table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*InsertVector.srv*]),
    [\- name: *string* #linebreak() \- properties: *Property[]* #linebreak() \- vector: *float[]*],
    [],
  ), table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*Property.msg*]),
    [\- key: *string* #linebreak() \- value: *string*],
  )),
) <data-weaviate-insert>

Finally, we provide a service that lets us query these vectors.

#figure(
  kind: "data",
  supplement: [Data Structure],
  caption: "Weaviate Query Vector Service",
  grid(columns: (auto, auto), inset: 5pt, table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*QueryVector.srv*]),
    [\- name: *string* #linebreak() \- limit: *int* #linebreak() \- vector: *float[]*],
    [\- results: *SearchResult[]*],
  ), table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*SearchResult.msg*]),
    [\- certainty: *float* #linebreak() \- properties: *Property[]*],
  )),
) <data-weaviate-query>

#pagebreak(weak: true)
== Task Design

A state machine will be used to implement the task requirements, it will be implemented using the following overarching
design:

#figure(image("../../images/theoretical-states/state_machine.png", width: 280pt), caption: [State machine]) <theoretical-state-machine>

But before we run the task, we need to prepare information about the arena and ensure the program can correctly identify
where it is. We provide setup instructions that detail how the user configures the start/end position in the arena, the
waiting position, the position of the seating area, the area of the waiting area, the name & favourite drink of the
host, and information required to reidentify the host (@theoretical-extract-information describes the steps necessary to
learn a new face, this will be described in more detail below). This can be provided through a configuration utility or
we can instruct the user to write information into specified files.

*Keeping track of information throughout task*

However, we still need to consider how we keep track of all of this information. To begin, we consider the persistent
information such as the arena information and the host's information. We can use the following data structure within the
parameter server:

#figure(kind: "data", supplement: [Data Structure], caption: "Data on Parameter Server", fletcher.diagram(
  node-stroke: 0pt,
  edge-stroke: 0.3pt,
  node((0, 0.38), "/receptionist"),
  edge((0, 0.38), (1, -0.05), "->"),
  node((1, 0), "/points"),
  edge("->"),
  node((2, 0), [/\<name\>: *float[]*]),
  node((1, 0.4), "/polygons"),
  edge((0, 0.4), (1, 0.4), "->"),
  edge("->"),
  node((2, 0.4), [/\<name\>: *float[]*]),
  node((1, 0.8), "/host"),
  edge((0, 0.4), (1, 0.9), "->"),
  edge("->"),
  node((2, 0.8), [/name: *string*]),
  node((2, 1.2), [/face: *float[]*]),
  edge((1, 0.8), (2, 1.3), "->"),
)) <data-task-parameter-server>

This information will persist through invocations.

#pagebreak()

The point and polygon names are described by the following enums:

#align(center, grid(
  columns: (auto, auto),
  inset: 5pt,
  [#figure(kind: "data", supplement: [Data Structure], caption: "Points enumeration", table(
      columns: (auto),
      inset: 5pt,
      align: horizon + center,
      table.header([\<\<enumeration\>\> #linebreak() *Points*]),
      [home #linebreak() wait_for_person #linebreak() seating_area],
    )) <data-points-enum>],
  [#figure(kind: "data", supplement: [Data Structure], caption: "Polygons enumeration", table(
      columns: (auto),
      inset: 5pt,
      align: horizon + center,
      table.header([\<\<enumeration\>\> #linebreak() *Polygons*]),
      [waiting_area],
    )) <data-polygon-enum>],
))

Next, we want to consider how to store ephemeral information such as the learned guest's name, description, position (in
the seating area), and favourite drink. First, we define two new data structures that represent the more complex
information such as their description and position, for which we

#align(center, grid(columns: (auto, auto), inset: 5pt, [
  #figure(kind: "data", supplement: [Data Structure], caption: "Guest description", table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*GuestDescription*]),
    [\- detection: *YoloDetection* #linebreak() \- features: *FeatureWithColour[]*],
  )) <data-guest-description>
], [#figure(kind: "data", supplement: [Data Structure], caption: "Guest position", table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*GuestPosition*]),
    [\- point: *geometry_msgs/Point* #linebreak() \- confidence: *float*],
  )) <data-guest-position>]))

Then we define a class that contains all of this information. We use dictionaries that map the guest's index to the
respective features. This class will be instantiated once and then passed through the state machine to all states so
that we can read/write this data at any time.

#figure(
  kind: "data",
  supplement: [Data Structure],
  caption: "Ephemeral state (persists through task)",
  table(
    columns: (auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*EphemeralState*]),
    [\- guest_descriptions: *int $->$ GuestDescription* #linebreak() \- guest_points: *int $->$ GuestPosition* #linebreak() \- guest_names:
      *int $->$ string* #linebreak() \- guest_drinks: *int $->$ string*],
  ),
) <data-ephemeral-state>

*Navigation to different beacons*

TIAGo's navigation stack can already handle planning around obstacles and looking in the correct direction while
navigating, such that we just need to send a target goal to the move_base server running on the robot.

#pagebreak(weak: true)
*Wait for a person to appear and approach them*

#wrap-content(
  [#figure(image("../../images/theoretical-states/wait_for_person.png", width: 100pt), caption: ["Wait for a person"]) <theoretical-wait-for-person>],
  [To implement this state, we need to position ourselves in front of the waiting area and then continuously fetch camera
  and depth information from the robot. There will be an object detection service that uses YOLOv8 (trained on COCO
  @cocodataset) to segment out the objects in the scene, the service will accept the camera image and return a response
  containing the detections (@data-yolo-detection). We can filter the list of detections where `name: person`.

  These segmentation mask(s) can be used to select points in the pointcloud (a 3D representation of the depth information)
  corresponding to the people in the scene. We can then filter the people positions for those who are in the waiting area
  we defined earlier. If there are one or more people (though we shouldn't ever see more than one person in the waiting
  area) then we can select the closest or any random person and approach them by planning to an unoccupied point within an
  area around them, then turn to face them.],
)

*Generate a description of a guest*

#wrap-content(
  [
    #figure(image("../../images/theoretical-states/describe_guest.png", width: 180pt), caption: ["Describe guest"]) <theoretical-describe-guest>
  ],
  [This state will take an image from the robot's camera and feed it into both the object segmentation service described
    earlier and a body part segmentation service, this new service will accept the camera image and feed it into BodyPix
    along with a requested set of body parts (we want to find the torso and head). The service will then return a set of
    segmentation masks corresponding to the list we requested.

    Now, we can combine the information we've gathered. First, we filter the object segmentation results for only people as
    described before, and now we can use the segmentation masks for each person to mask out different body parts, and those
    new masks are now in turn used to mask the original image to find colour information. We do this as the body part
    segmentation model won't give us enough information to distinguish between different people so we have to do it
    ourselves. At this point, we can now also determine the colour of the torso by taking the mean colour in the segmented
    area and finding the closest known labeled colour.

    Then for each person, we can process the segmented head to extract additional information such as if they are wearing
    glasses or a hat, or find the colour of their hair. To do this, we can take an existing dataset that labels and segments
    these characteristics and train it using a tool such as PyTorch. We package this into a new service that accepts a
    picture of someone's face, segments it, tries to determine the colour of the segmented image and spits out a list of
    features (@data-feature).

    Combining all of this information, we can now extract the colour of someone's torso, the colour of their hair, whether
    they are wearing glasses, and whether they are wearing a hat. We take all of the data gathered and format it as
    @data-guest-description which can be stored in the ephemeral information passed between states (@data-ephemeral-state).],
)

*Ask guest for their favourite drink and their name*

#wrap-content(
  [#figure(image("../../images/theoretical-states/ask_guest.png", width: 110pt), caption: ["Ask guest for information"]) <theoretical-ask-guest>],
  [This state is quite simple, it repeatedly asks the guest their favourite drink until they're understood, and then
    likewise prompts the guest for their name until they're understood. To talk to the user, we use the TIAGo's onboard
    text-to-speech service which we can simply call with a string to be read out.

    To implement it, we will deploy two new services: one that provides speech recognition and another for dialog intent
    recognition. The speech recognition service will implement an action server that listens on a given preconfigured
    microphone, feeds the audio data in chunks into Whisper speech-to-text model, and returns a string containing the
    transcribed speech after a phrase is detected. If it fails to recognise any speech, it will prompt the user to repeat
    and continue to generate a new transcription.

    This transcription will be sent to the other new service that passes the given string to a custom RASA model that is
    trained to understand responses such as "My name is Adel." (intent: name) and "orange juice" (intent: drink). The given
    response will be processed by the state and if the intent is unclear, the system will ask the user to try again.],
)

*Extract information for reidentification*

#wrap-content(
  [
    #figure(
      image("../../images/theoretical-states/extract_information.png", width: 110pt),
      caption: ["Collect reidentification data"],
    ) <theoretical-extract-information>
  ],
  [This state will take an image from the robot's camera and, like the description state, segment both the full body and
    body parts using the appropriate models. We specifically segment the closest person's head and then convert this image
    into an embedding that we can later query. To convert the image to an embedding, we need to vectorise the face features,
    we can use many different models described earlier in the face reidentification literature section.

    This state will run concurrently when we speak with the guest and it will keep gathering data until it is preempted.
    This is done to gather as many embeddings of the person as possible and as such produce a much more reliable dataset.],
)

#pagebreak(weak: true)
*Guide guests to an empty seat and ask them to sit down*

#wrap-content(
  [
    #figure(image("../../images/theoretical-states/guide_guest.png", width: 65pt), caption: ["Guide guest"]) <theoretical-guide-guest>
  ],
  [This state will make the robot tell the guest to follow it to the seating area. Once the robot arrives at the seating
    area, it will start sweeping the camera and will take a camera image with depth information in each direction to try to
    detect an empty chair. To sweep the camera, we can send a 'motion' to the TIAGo's head controller, we define target
    destinations where we are looking slightly left, slightly right, and centre. To detect the chairs and their positions,
    we use the same strategy as when we detect and describe a guest.

    However, we specifically want to find empty chairs, so we opt to filter for both person and chair detections from the
    object detection service. We can now omit all of the chair masks that intersect person masks, hence finding all of the
    empty chairs.

    To direct the guest to the chair, we will turn to it and point at it with a predefined gesture using the TIAGo's arm.
    Finally, the robot will tell the guest to sit down in the given seat. If we've only met one guest so far, we need to the
    waiting area and meet the next guest.],
)

*Reidentify guests*

At this point, the robot will be in the seating area and must reidentify all of the guests. We cannot simply remember
where we sat the guests down because they may move at any point, and for RoboCup competition runs, guests have been
instructed to move after the robot leaves.

We use the same strategy as described before to generate a face image (@theoretical-extract-information), we can then
vectorise this, and query the embeddings database we populated earlier through the task to determine which face
corresponds to each guest, and hence find the centroids of each person.

*Describing the first guest to the second guest*

For this state, we use the guest information from the reidentification step prior. The robot must turn to the second
guest and speak details found earlier about the first guest. This state could be made more natural by doing a formal
introduction between the two guests, turning to face each other, and for example, saying the other's name.

*Ending the task*

To end the task, the robot will navigate back to the starting position and announce that the task has ended.