#import "@preview/lovelace:0.2.0": *
#import "../../../lib/admonitions.typ": todo
=== Storing Data & Setup

The project deals with a variety of different data that is collected before and during the task, different approaches
are necessary depending on the type of data we are handling.

==== Persistent arena information

We use @data-task-parameter-server described earlier to represent persistent arena information, this information lives
in the parameter server. To generate the data in the first place, we provide the user with a configuration CLI built
using the typer Python library @pypi_typer. This library was chosen as it provides a very simple, straightforward, and
scalable way to build CLI utilities:

```python
app = typer.Typer() # after defining the Typer application,
@app.command() # we can define a new command
def hello(): # this is equivalent to "python path/to/file.py hello"
  print("hello!")

app() # before we exit, we just invoke the app
```

So the CLI defines the commands: `load [name]`, `save [name]`, `points [set|get] [name]`,/* @typstyle off */`polygons [set|get] [name] [count] (topic)`,
and `learn-host [name]`. To begin, load and save simply copy the configuration in the parameter server from/to a JSON
file relative to the project. We use `json.loads/dumps` from the Python standard library to convert and we can use
`rospy.set|get_param('/receptionist'[, obj])` to set/load the entire state at once.

The points command works by waiting for the next published robot pose from TIAGo, published on `/amcl_pose`, and saves
it as-is to the parameter server under `/receptionist/points/[name]`. This records the robot's current position and
rotation so it can be used to return to this given position in the future. The user is asked to save points for the home
point, the point where we wait for a new person, and the waiting area.

The polygons command works by waiting for $n$ published points on a given topic (assumed to be `/clicked_point` by
default) and collects them before saving them to the parameter server under `/receptionist/polygons/[name]`. The user is
instructed to use a utility such as RViz @rizz in which the user may select the "Publish Point" option in the toolbox
and then select points in the map.

#algorithm(
  caption: [Polygon Selection],
  pseudocode-list[
    - *input:* topic: _string_, $n$ count: _int_
    - *output:* polygon: _float[][]_
    - *requires:* rospy: _ROS Python Library @rospy/* @typstyle off */_
    + polygon $<-$ []
    + *for* $i = 0..n$ *do*
      + $p <-$ rospy.wait_for_message(topic, geometry_msgs/Point) #comment[wait for user input...]
      + polygon $<<- p_(x y)$ #comment[we want to ignore the z axis as we are looking for areas in the 2D map only]
    + *end*
  ],
)

Finally, the learn host command will save the guest's name and face vectors into `/receptionist/name|face`. To learn the
host's face, we use the detect faces state machine discussed later in @detect-faces to generate vectors that we store in
the parameter server.

==== Epheremal guest information

For storing guest information during the execution of the task, such as guest names, favourite drink, positions, and
descriptions, we use @data-ephemeral-state. An empty copy of this data structure is initialised alongside the state
machine on invocation and is then passed through to every state:

```python
sm = smach.StateMachine() # when we define our state machine,
state = EpheremalState() # we also define our state,
# ...
sm.add(MyStateMachine(state)) # and we pass it through to any sub-state(-machine)
```

However, information such as face data is stored using the Weaviate vector database described earlier as it will also
provide a way to query this information efficiently later.

