#import "../../../lib/diagrams.typ": blob, fletcher, edge, node
#import "@preview/drafting:0.2.0": margin-note
#import "@preview/wrap-it:0.1.0": wrap-content
#import "@preview/lovelace:0.2.0": *
#pagebreak(weak: true)
=== State Machine

During the development of the project, I found that I had to make certain changes to the planned state machines.
@state_machine describes the final overarching state machine that I ended up implementing.

#figure(fletcher.diagram(
  node-stroke: 0.5pt,
  edge-stroke: 0.7pt,
  blob((0, 0), "Start", tint: gray),
  edge("-->", "(sim)"),
  blob((1, 0), "Setup Sim"),
  edge("->"),
  blob((0, 1), "Enable Head"),
  edge((0, 0), (0, 1), "->"),
  edge("->"),
  blob((0, 2), "Navigate to start"),
  edge("->"),
  blob((-1, 2), "Speak \"Starting task\""),
  edge("->"),
  blob((-1, 3), "Run for two guests", tint: yellow),
  edge("->", "i=1..2"),
  blob((0, 3), "Navigate to waiting area"),
  edge("->"),
  blob((1, 3), "Reset Head"),
  edge("->"),
  blob((1, 4), "Wait for a person", tint: red),
  edge("->"),
  blob((0, 4), "Meeting the guest", tint: red),
  edge("->"),
  blob((0, 5), "Enable Head"),
  edge("->"),
  blob((0, 6), "Seating guest", tint: red),
  edge((0, 6), (-0.5, 6), (-1, 3), "->", bend: 100deg),
  blob((-1, 6), "Navigate to seats"),
  edge((-1, 3), (-1, 6), "-->", "i>2"),
  edge("->"),
  blob((-1, 7), "Introducing guests", tint: red),
  edge("->"),
  blob((0, 7), "Navigate to end"),
  edge("->"),
  blob((1, 7), "Speak ``Finished task''", tint: gray),
), caption: "Overarching state machine for the task") <state_machine>

==== Preparative Setup

Before we start the state machine, we need to prepare a few things. We begin by creating a new collection in the vector
database with the name "Face" and we request that if the collection already exists, it is cleared. Afterwards, we check
for the presence of host information within the parameter server. If information is present, we upload all of the
vectors using the procedure described in @uploading-to-weaviate.

#pagebreak(weak: true)
==== Simulation Setup

#wrap-content(
  figure(
    caption: [Automatic placement #linebreak() of objects in simulation],
    image("../../../images/screenshots/Screenshot from 2024-03-22 13-23-47.png", width: 140pt),
  ),
  [Initially, the entire project was built and tested within the Gazebo simulator. Manually setting up the scenario was
  cumbersome and annoying, and trying to set up a custom world was also infeasible due to the way TIAGo's simulation
  launch files were set up. So the solution was to investigate whether Gazebo offered any way to automate the spawning and
  deletion of objects as well as whether it could allow resetting the simulation as a whole.

  To spawn new objects, Gazebo accepts SDF (Simulation Description Format) models, which are plain XML files that describe
  the mesh, physics, and other properties of the model. It accepts these on the `/gazebo/spawn_sdf_model` service along
  with a reference frame, name, and an initial pose. So we make a list of all the objects we want, e.g. 2 guests, 3
  chairs, along with the target initial poses within the world. We can then iterate through this list and call the service
  to spawn them.

  To then delete the object, we call `/gazebo/delete_model` with the name we specified earlier. To make things easier to
  later reset, all of the models we spawn are prefixed with `receptionist_`.

  Each time the task is started (and we specify this when launching), the first state the machine starts at is the
  simulation setup. In this state, we first reset the existing simulation by waiting for the next message on the topic
  `/gazebo/model_states` which provides us with a list of models in the world, we filter for those starting with our
  prefix and subsequently delete them, and finally we iterate through the list of predefined models and spawn all of them.

  Gazebo also offers a way to reset the simulation directly, by calling `/gazebo/reset_simulation` service but this comes
  with two caveats making it unsuitable: models (spawned since load) do not despawn and the robot loses its localisation.],
)

==== Head Manager

Head manager is a node that runs on the TIAGo robot that controls the looking direction of the head and automatically
looks in the direction of travel/rotation when navigating. This allows us to automatically satisfy the direction of
travel requirement however we must take care to disable/enable the head manager as appropriate, as it causes the robot
to naturally look around when idle which is unintended behaviour when we need to look at guests.

To achieve this, we provide two states "Enable Head" and "Disable & Reset Head". The head manager allows us to control
whether it is active via an action server that prevents it from working when there is an active goal, so when we disable
it, we must send a disable action to `/pal_head_manager/disable`. But the head may still be looking in some random
direction, so we also send a goal to "play motion" which is an action server running on the TIAGo robot that can play
predefined motions. In our launch file, we loaded several motions onto the parameter server, we use "look_centre" which
resets the head to look straightforward.

To then re-enable the head, we can cancel all goals sent to the action server.

==== Navigation to beacons

Navigation is relatively straightforward, we load the points defined earlier from the parameter server and then send an
action to the `/move_base` action server (navigation stack) with the desired pose. If navigation fails due to poor
localisation, the robot will automatically enter recover behaviour which includes actions such as spinning the robot in
order to collect information about the surroundings and relocalise.

It is unlikely the navigation action will catastrophically fail within these constrained environments, so any failure is
handled as a total task failure.

==== Text-to-speech

Text-to-speech is also relatively straightforward as this is already provided by the robot, we simply have to send a
goal to the `/tts` action server with our desired text and wait for completion.

==== Running for two guests

To run a set of states twice, we use an Iterator container from SMACH. It is composed of two parts, the iterator itself
which holds a Python iterator and provides the current value through a chosen key in the userdata and an inner container
in which we build our desired states.

To implement this state, we create an iterator that runs over the range $1 <= i < 3$, we ask that the current value of $i$ is
provided through the `index` key on the userdata, and we build the sub-state machine as in @state_machine in the
container. Once the iterator is exhausted, we transition to the next phase and go to the seating area.

==== Waiting for a person

This is a relatively straightforward state that first waits for the next PointCloud information from the robot, sends it
to the object detection `/detect3d` service discussed earlier, filters the results by those whose points are within the
predefined waiting area, and finally has a successful outcome if it finds someone or a failure outcome if it does not.

#figure(fletcher.diagram(
  node-stroke: 0.5pt,
  edge-stroke: 0.7pt,
  blob((0, 0), "Start", tint: gray),
  edge("->"),
  blob((1, 0), "Get PCL"),
  edge("->", "{pcl}"),
  blob((2, 0), "Detect Objects 3D"),
  edge("->", "{objects}"),
  blob((3, 0), "Filter In Area"),
  edge("->"),
  blob((4, 0), "End", tint: gray),
), caption: "State machine for waiting for a new person") <state_wait_for_person>

In order to filter for detections that are in the area, we use the shapely Python library @pypi_shapely which provides
algorithms for polygon and point intersections. We load the list of points for the waiting area and convert it to a
shapely Polygon. Then we take the x,y components of the objects' points, convert them to shapely Points, and call
`Polygon.contains(point)` for each.

This state lives in the lasr_skills package @lasr-skills.

#pagebreak(weak: true)
=== SM: Meeting the guest

To run multiple state machines at once, we use a Concurrence container from SMACH. Once all the sub-state machines have
reached a successful outcome, the concurrence as a whole will succeed. It may likewise fail and preempt all other
states.

For the purposes of this task, we want to indefinitely collect face reidentification data until we finish generating a
description and talking to the guest. To achieve this, we must provide a mechanism for preempting the infinite loop and
allowing the state machine to finish. SMACH provides a way to add a callback on whether the state machine should be
preempted based on the termination of its children. We implement the following:

#algorithm(caption: [Child Termination Callback], pseudocode-list[
  - *input:* states: _str $->$ str_
  - *output:* terminate: _bool_
  + terminate $<- bot$ #comment[allow states to continue running by default]
  + *if* states.DESC.succeeded *and* states.TALK.succeeded *then*
    + terminate $<- top$ #comment[signal that we want to preempt all other states]
  + *end*
])

#figure(fletcher.diagram(
  node-stroke: 0.5pt,
  edge-stroke: 0.7pt,
  blob((0, 0), "Start", tint: gray),
  edge("->", "DESC"),
  blob((-1, 1), "Create description", tint: red),
  // edge("->", "{people}"),
  // blob((-1, 2), "Save description"),
  edge((-1, 1), (-1, 5.5), (-0.8, 6), (0, 6), "->", "{desc}"),
  blob((0, 6), "Wait for all to complete and save data"),
  edge("->"),
  blob((0, 7), "End", tint: gray),
  edge((0, 0), (0, 1), "->", "TALK"),
  blob((0, 1), "Speak ``Name?''"),
  edge("->"),
  blob((0, 2), "Listen for an answer", tint: red),
  edge("->", "{name}"),
  blob((0, 3), "Speak ``{name}, F. Drink?''"),
  edge("->"),
  blob((0, 4), "Listen for an answer", tint: red),
  edge("->", "{drink}"),
  blob((0, 5), "Speak ``{drink}''"),
  edge((0, 5), (0, 6), "->"),
  edge((0, 0), (1, 1), "->", "DET"),
  blob((1, 1), "Detect 1 face(s)", tint: red),
  edge("->", "{vec}"),
  blob((1, 2), "Insert vector"),
  edge((1, 2), (2, 2), (2, 1), (1, 1), "->"),
  edge((1, 2), (1, 5.5), (0.8, 6), (0, 6), "-->", "preempted"),
), caption: "State machine for meeting and learning about a new guest") <state_meeting_guest>

==== Facilitating preemption

The loop we've created will not automatically understand when it must break out and complete execution. So we introduce
a new state, we call it the PreemptionHandler, which has two outcomes: continue or preempt. It is implemented as
follows:

#algorithm(caption: [PreemptionHandler State], pseudocode-list[
  - *output:* outcome: _str_
  + *if* self.preempt_requested() *then* #comment[ask SMACH if we need to preempt]
    + self.service_preempt() #comment[notify SMACH that the preemption has been handled]
    + outcome $<-$ "preempted" #comment[notify our code that we must stop execution of this state]
  + *else*
    + outcome $<-$ "continue"
  + *end*
])

==== Detecting faces <detect-faces>

This is also quite a simple state: we fetch the next image from the camera, send it to the DeepFace service to find all
of the faces in the image, and finally, we may optionally check the count and fail if it is not what we expected. The
check count state simply asserts that the length of `faces` is correct.

#figure(fletcher.diagram(
  node-stroke: 0.5pt,
  edge-stroke: 0.7pt,
  blob((0, 0), "Start", tint: gray),
  edge("->"),
  blob((1, 0), "Get Camera Image"),
  edge("->", "{img}"),
  blob((2, 0), "DeepFace /detect"),
  edge("->", "{faces}"),
  blob((3, 0), "Check Count"),
  edge("->"),
  blob((4, 0), "End", tint: gray),
), caption: "State machine for detecting faces") <state_detect_faces>

==== Inserting vectors <uploading-to-weaviate>

This state takes the index and vector from userdata and uploads it to the vector database using the
`/database/vectors/weaviate/insert` service with the following format:

#table(
  columns: (auto, auto, auto),
  table.header("Collection Name", "Properties", "Vector"),
  ["Face"],
  [`{"guest": userdata.index}`],
  [userdata.vector],
)

#pagebreak(weak: true)
==== Creating a description of the guest

#figure(fletcher.diagram(
  node-stroke: 0.5pt,
  edge-stroke: 0.7pt,
  blob((0, 0), "Start", tint: gray),
  edge("->"),
  blob((0, 1), "Get camera image"),
  edge("->", "{img_msg}"),
  blob((0, 2), "Convert image to numpy array"),
  edge("->", "{img}"),
  blob((0, 3), "Segment using YOLO"),
  edge("->", "{detection}"),
  blob((0, 4), "Extract features"),
  edge("->"),
  blob((0, 5), "End", tint: gray),
  edge((0, 2), (0.8, 2), (1, 3), "->", "{img}"),
  blob((1, 3), "Segment using BodyPix"),
  edge((1, 3), (0.8, 4), (0, 4), "->", "{masks}"),
), caption: "State machine for generating a description of the guest") <state_describe_guest>

This state takes the latest image from the robot's camera and segments it using both object detection and body
segmentation services. For the object segmentation service, we request only people so we receive back a list of
segmentation masks for each person in the scene. For the body segmentation service, we request the torso and face.

Next, for each person, we look at each body part mask and either extract the colour for the torso or perform additional
inference using the face features service for the face mask. We need to also segment each body part mask using the
person mask to avoid processing the wrong part of the image that isn't relevant to the person we want. In essence, it
works as follows:

#algorithm(
  caption: [Extract Features State],
  pseudocode-list[
    - *input:* detection: _YoloDetection[]_, masks: _BodyPixMask[]_, image: _numpy.array_
    - *output:* people: _YoloDetection $->$ FeatureWithColour[]_
    - *requires:* numpy: _numpy @pypi_numpy/* @typstyle off */_, cv2: _OpenCV2 @pypi_opencv/* @typstyle off */_, colours: _colour_estimation_
    + people $<-$ []
    + height, width, channels $<-$ image.shape #comment[determine the size of the image]
    + *for* person *in* detection *do*
      + mask_image $<-$ numpy.zeros((height, width)) #comment[create an empty image]
      + cv2.fillPolygon(mask_image, person.xyseg, color=white) #comment[fill in the outline of person]
      + mask_bin $<-$ mask_image $> 128$ #comment[create a binary mask using 50% brightness as threshold]
      + features $<-$ []
      + *for* index, part_mask *in* masks *do*
        + part_mask[mask_bin == 0] $<-$ 0 #comment[remove mask areas that belong to other people]
        + *if* index *is* 0 *then* #text(gray)[#raw("torso mask")]
          + median $<-$ numpy.median(image[part_mask == 1])
          + features $<<-$ FeatureWithColour("torso", colours.nearest_name(median))
        + *else* #text(gray)[#raw("head mask")]
          + features $<<-$ #raw("result from face feature service")
        + *end*
      + *end*
    + *end*
  ],
)

==== Listening for answers from the guest

In this state, we use the Whisper transcription action server to wait for a new phrase being spoken by the user, this is
sent to RASA to figure out the intent of what the user is saying, and then we can try to interpret the given intent with
the detected phrase entities. If we are in a simulation, we instead simulate a waiting period of a few seconds and then
pass example speech through userdata. If we fail to understand what the user says, we tell them to try again.

#figure(fletcher.diagram(
  node-stroke: 0.5pt,
  edge-stroke: 0.7pt,
  blob((0, 0), "Start", tint: gray),
  edge("->"),
  blob((0, 1), "Transcribe speech"),
  edge("->", "{text}"),
  blob((0, 2), "Parse speech"),
  edge("->", "{intent}"),
  blob((0, 3), "Interpret speech"),
  edge("->"),
  blob((0, 4), "End", tint: gray),
  edge((0, 3), (-1, 3), "-->", "(fail)"),
  edge((0, 2), (-0.8, 2), (-0.8, 3), "-->", "(fail)"),
  edge((0, 1), (-0.9, 1), (-0.9, 3), "-->", "(fail)"),
  blob((-1, 3), "Speak ``try again''"),
  edge((-1, 3), (-1, 0), (0, 0), "->"),
  edge((0, 0), (1, 0), "-->", "(sim)"),
  blob((1, 0), "Simulate Wait"),
  edge("->"),
  blob((1, 1), "Example Speech"),
  edge((1, 1), (0.8, 2), (0, 2), "->", "{text}"),
), caption: "State machine for listening for a guest's spoken answer") <state_listen>

Here's an example of how the intent is interpreted:

#algorithm(caption: [Interpret Intent (for person name)], pseudocode-list[
  - *input:* intent: _JSON Object_
  - *output:* name: _str_
  + *assert* intent['intent']['name'] == "person_name" #comment[ensure we are looking at correct intent]
  + *assert* "name" *in* intent['intent'] #comment[ensure a name has been detected by RASA]
  + *assert* *length* intent['entities']['name'] *is* 1 #comment[expect there to only be one name]
  + name $<-$ intent['entities']['name'][0]
])

#pagebreak(weak: true)
=== SM: Seating the guest

This state consists of previously defined states with one new sub-state machine.

#figure(fletcher.diagram(
  node-stroke: 0.5pt,
  edge-stroke: 0.7pt,
  blob((0, 0), "Start", tint: gray),
  edge("->"),
  blob((0, 0.8), "Speak ``Please follow me''"),
  edge("->"),
  blob((0, 1.6), "Navigate to seating area"),
  edge("->"),
  blob((0, 2.4), "Disable & reset head"),
  edge("->"),
  blob((0, 3.2), "Speak ``This is {guest_name}''"),
  edge("->"),
  blob((0, 4), "Identify empty chair", tint: red),
  edge("->"),
  blob((0, 4.8), "Wait for 3s"),
  edge("->"),
  blob((0, 5.6), "Enable head"),
  edge("->"),
  blob((0, 6.4), "End", tint: gray),
), caption: "State machine for seating a guest") <state_seating_guest>

To identify a chair, we segment out all the chairs and people in the scene and find those that have nobody overlapping
them. Any failure results in the robot asking guests to sit anywhere.

#figure(fletcher.diagram(
  node-stroke: 0.5pt,
  edge-stroke: 0.7pt,
  blob((0, 0), "Start", tint: gray),
  edge("->"),
  blob((0, 0.8), "Get camera image"),
  edge((0, 0.8), (1.1, 0.8), (1.1, 4), "-->", "(fail)"),
  edge("->", "{img}"),
  blob((0, 1.6), "Segment with YOLO"),
  edge((0, 1.6), (1, 1.6), (1, 4), "-->"),
  edge("->", "{detection}"),
  blob((0, 2.4), "Pick empty chair"),
  edge((0, 2.4), (0.95, 2.4), (0.95, 4), "-->"),
  edge("->", "{point}"),
  blob((0, 3.2), "Look to point"),
  edge((0, 3.2), (0.9, 3.2), (0.9, 4), "-->"),
  edge("->"),
  blob((0, 4), "Speak ``sit in chair I'm looking at''"),
  edge("->"),
  blob((0, 4.8), "End", tint: gray),
  blob((1, 4), "Speak ``sit in any chair''"),
  edge((1, 4), (1, 4.8), (0, 4.8), "->"),
), caption: "State machine for identifying an empty chair") <state_identify_chair>

#algorithm(caption: [Pick Empty Chair], pseudocode-list[
  - *input:* detection: _YoloDetection[]_
  - *output:* point: _geometry_msgs/Point | None_
  - *requires:* shapely: _Shapely @pypi_shapely/* @typstyle off */_
  + people $<-$ []
  + *for* object *in* detection *do*
    + *if* object.name *is* "person" *then* #comment[generate polygon data structures for all people in frame]
      + people $<<-$ shapely.Polygon(object.xyseg)
    + *end*
  + *end*
  + *for* object *in* detection *do*
    + *if* object.name *is* "chair" *then*
      + polygon $<-$ shapely.Polgyon(object.xyseg)
      + intersects $<- bot$
      + *for* person *in* people *do*
        + *if* person.intersects(polygon) *then*
          + intersects $<- top$ #comment[someone is sitting in the chair, so ignore it]
        + *end*
      + *end*
      + *if* intersects *is* $bot$ *then*
        + point $<-$ object.point #comment[pick this chair's centroid as the point to look at]
      + *end*
    + *end*
  + *end*
])

==== Look to point <look_to_point>

Look to point is a state provided by the lasr_skills package @lasr-skills. It is a primitive wrapper around the action
server `/head_controller/point_head_action` which accepts a frame to compute relative to and a target to look at. The
goal is defined as so: (with sample values)

#figure(
  kind: "data",
  supplement: [Data Structure],
  caption: "Point Head Action Implementation",
  table(
    columns: (auto, auto),
    inset: 5pt,
    align: horizon + left,
    table.header([*PointHeadGoal.msg*], [Values]),
    [\- pointing_frame: *string* #linebreak() \- pointing_axis: *geometry_msgs\/Point* #linebreak() \- max_velocity: *float* #linebreak() \- target.header:
      *std_msgs/Header* #linebreak() \- target.point: *geometry_msgs/Point*],
    [head_2_link #linebreak() $[1.0, 0.0, 0.0]$ #linebreak() $1.0$ #linebreak() frame_id=map #linebreak() userdata.point],
  ),
) <data-point-head>

#pagebreak(weak: true)
=== SM: Introducing guests

This is the final phase of the task, during which we find where everyone is and then describe the first guest to the
second guest by turning to look at where the second guest is and recalling the description generated for the first guest
earlier.

#figure(fletcher.diagram(
  node-stroke: 0.5pt,
  edge-stroke: 0.7pt,
  blob((0, 0), "Start", tint: gray),
  edge("->"),
  blob((0, 1), "Disable head"),
  edge("->"),
  blob((0, 2), "Run for each direction", tint: yellow),
  edge("-->", "exhausted"),
  blob((0, 4), "Select second guest"),
  edge("->", "{point}"),
  blob((0, 5), "Look to point"),
  edge("->"),
  blob((0, 5.8), "Describe first guest"),
  edge("->"),
  blob((0, 6.6), "Enable head"),
  edge("->"),
  blob((0, 7.4), "End"),
  edge((0, 2), (1, 2), "->"),
  blob((1, 2), "Select direction"),
  edge("->"),
  blob((1, 3), "Turn camera"),
  edge("->"),
  blob((1, 4), "Identify guest positions", tint: red),
  edge((1, 4), (0.5, 4), (0, 2), "->"),
), caption: "State machine for introducing guests to each other") <state_seating_guest>

==== Turning the camera

We use the play motion action server to send a goal to look left, right, and then to the centre. (we sweep across from
left to right to then avoid having to correct from a side to the centre and add complexity) Each direction also
overlaps, which means guests should typically be scanned at least twice if they're in front of the robot and allows
people sitting to the side of the robot to have a more accurate front-facing scan of their face which increases our
accuracy.

#pagebreak(weak: true)
==== Identifying guest positions

This state detects people in 3D much like we did earlier when finding empty chairs but also finds the faces present in
the frame. The face vector is queried against the vector database using the `/database/vectors/weaviate/query` service.
Each item in the list of results is a potential match with the confidence and properties (which includes the guest
index). Since this state runs multiple times, this allows us to pick the highest confidence results for each guest.

#figure(fletcher.diagram(
  node-stroke: 0.5pt,
  edge-stroke: 0.7pt,
  blob((0, 0), "Start", tint: gray),
  edge("->"),
  blob((0, 0.8), "Detect People 3D"),
  edge("->", "{detection}"),
  blob((0, 1.6), "Detect Faces"),
  edge("->", "{faces}"),
  blob((0, 2.4), "Run for each face", tint: yellow),
  edge("->", "{face}"),
  blob((1, 2.4), "Query Vector"),
  edge("->", "{results}"),
  blob((1, 3.2), "Save Position"),
  edge((1, 3.2), (0.5, 3.2), (0, 2.4), "->"),
  edge((0, 2.4), (0, 3.2), "-->", "exhausted"),
  blob((0, 3.2), "End", tint: gray),
), caption: "State machine for identifying guest positions") <state_guest_positions>

#algorithm(
  caption: [Save Guest Position],
  pseudocode-list[
    - *input:* face: _Embedding_, detection: _YoloDetection[]_, result: _SearchResult[]_
    - *requires:* shapely: _Shapely @pypi_shapely/* @typstyle off */_, state: _EphemeralState_
    + face_area $<-$ shapely.Polygon([ [face.x, face.y], [face.x + face.w, face.y], [face.x + face.w, face.y + face.h],
      [face.x, face.y + face.h] ]) #comment[convert xywh to polygon]
    + result $<-$ results[0] #comment[results is sorted by confidence as it comes from database]
    + confidence $<-$ result.confidence
    + *for* person *in* detection *do*
      + polygon $<-$ shapely.Polgyon(person.xyseg)
      + *if* polygon.intersects(face_area) *then* #comment[this face corresponds to this person]
        + current_confidence $<-$ state.get_guest_point_confidence(result.index)
        + *if* confidence > current_confidence *then*
          + state.set_guest_point(person.point) #comment[take this as the new assumed position of the guest]
          + state.set_guest_point_confidence(confidence) #comment[update best confidence]
  ],
)

==== Look at the second guest

In this state, we re-use @look_to_point, but instead of simply passing the position of the detected person, we first
update the $z$ component of the point to be $1.0$ which means the robot will look roughly eye-level for someone sitting
down on a chair.

==== Describe the first guest

We use the ephemeral state to populate information about the guest into the following:

#figure(
  caption: "String Interpolation",
  [
    Hi #box(fill: lime, outset: 2.5pt, radius: 3pt)[guest 1], #box(fill: aqua, outset: 2.5pt, radius: 3pt)[guest 2] is here
    with you, they are wearing a #box(fill: orange, outset: 2.5pt, radius: 3pt)[torso colour] top, they have #box(fill: purple, outset: 2.5pt, radius: 3pt)[#text(white)[hair colour]] hair,
    they #box(fill: eastern, outset: 2.5pt, radius: 3pt)[#text(white)[are / are not]] wearing glasses, and they #box(fill: maroon, outset: 2.5pt, radius: 3pt)[#text(white)[are / are not]] wearing
    a hat.
  ],
)