#pagebreak(weak: true)
== Approaches to Testing

This project follows a concise testing methodology, functional requirements (@func) are proved through running and
recording the results of the task on the real robot and in simulation, while non-functional requirements (with the
exception of N1, N2) are qualitatively assessed using specially designed experiments that measure how well a certain
aspect performs.

=== Testing the whole task

Running the task follows a simple procedure (which will be detailed more thoroughly in Appendix @user-manual). We must
first start all of the required services for the intended configuration. In simulation, we omit loading the
transcription server (responses are mocked) and load a mock text-to-speech server. Once it is confirmed they are all
running, we may load or create the appropriate configuration for the arena, and finally run the task.

To be able to later assess the results: The outputs from all relevant topics are saved using the rosbag utility (this
includes inference results, text-to-speech goals, and camera information). The output of the task is logged to a local
file. A screen cast is taken of the machine running the task. And if running on the robot, a video recording is taken of
the physical space.

=== Experiments

The project includes three 'scripts' (Python modules that either directly interface with various services to emulate
parts of the task or invoke parts of the state machine we've built) that provide data to qualitatively assess
requirements N3 through N5 (@non-func).

Unit 02 addresses requirement N3 "robot can reliably detect and reidentify faces": this script creates three collections
in the vector database into which we insert 1, 2, or 5 faces for each subject we involve from which to learn from. We
can then use the remainder of the face images for each subject to determine whether they can be reidentified and with
what confidence. #linebreak() *Dataset collected:* 5 subjects with 15 images each

Unit 03 addresses requirement N4 "reliably recognise speech and intent": this script reads in a given set of audio clips
with their associated spoken phrases transcribed by hand and feeds them into various speech-to-text models available
within this project. The resulting phrases are fed into the custom RASA assistant to see if the correct intent and
information can be extracted. We can calculate the success rates for both components. #linebreak() *Dataset collected:* 112
spoken phrases, 50 of which contain background noise representing a busy coffee shop, 12 of which are phrases that
aren't included in the training set for the assistant.

Unit 04 addresses requirement N5 "reliably generate selected characteristics": this script takes in a given folder of
pictures of people with associated metadata (known torso colour, hair colour, hat presence, glasses presence) and
executes the describe guest (@state_describe_guest) state machine. The results can be compared to the actual known
information to calculate the success rate of each feature extraction. #linebreak() *Dataset collected:* 30 unique images
of people with associated metadata; resized to 640px

For completeness, Unit 01 was a script used for testing face reidentification during development but it is not an
appropriate source of data.