== Development tooling and environments used

For this project, the editor of choice was Visual Studio Code @vscode with the catkin_tools @catkin_tools build system.
While Visual Studio Code is not necessarily considered an IDE, it can still work like a traditional IDE and provides
some additional useful features that were taken advantage of. VS Code still provides language server features through
extensions and offers the ability to remotely work on another machine through SSH. catkin_tools is the successor to
catkin_make @catkin_make which itself is just a thin wrapper around cmake, but it additionally provides functionality
such as isolated and parallel builds out of the box, which significantly reduces the time to build the project.

Throughout the project, different environments were used depending on changing requirements throughout the development
process, this section describes what was used and justifies why it was most appropriate.

=== Environment 1: Virtual machine with ROS Noetic

At the start of this project, a virtual machine was configured on a private server that could then be accessed remotely
through SSH and consequently allowed development through VS Code. To achieve this, Ubuntu 20.04 was used as the
operating system, and then ROS Noetic and the TIAGo public workspace were installed on top. Any additional dependencies
were also directly installed into the virtual machine. If a graphical environment was required, one could connect to the
machine over VNC running on the host machine.

This solution allowed development to take place anywhere regardless of the operating system in use or the hardware
configuration of any one computer it was accessed from.

=== Environment 2: Bare metal with Apptainer

Apptainer @apptainer is a program that allows us to build and run containers. Containers are essentially self-contained
images usually containing entire operating systems in themselves, or sometimes just an application with all the
dependencies required to execute it. PAL Robotics provides us with built containers that include the correct version of
Ubuntu, ROS Noetic, and the dependencies to run the TIAGo simulator or otherwise interact with a TIAGo robot. We can use
Apptainer to bootstrap from this image and add our dependencies on top.

The resulting container @robocup_container has a few notable changes including Python 3.9 and Python 3.10 which are not
available by default but we need for certain dependencies due to their strict version requirements. Various Python
packages such as numpy, scipy, tensorflow, etc are dependencies to other Python packages we will later install or use
within our workspace. An "overlay workspace" that includes additional ROS packages that we rely on.

A notable ROS package that we include in the overlay is catkin_virtualenv @catkin_virtualenv which allows us to directly
bundle any Python requirements we need within individual ROS packages via the use of Python virtual environments. This
is essential to allow us to use a specific version of Python where libraries necessitate it, for example, the
ultralytics @Jocher_YOLO_by_Ultralytics_2023 package requires at minimum Python 3.9, whereas the container uses Python
3.8 by default.