#import "../lib/admonitions.typ": todo

== Experimental Results

=== Findings from full task runs

The following findings are proven by the runs on the actual robot and in the simulation that is included in the
supplementary presentation video.

The robot successfully navigates between predefined points in the arena (T1) without failure. The robot can identify
when a person is or is not in the waiting area and does not move preemptively even if a person is visible in the
background (T2). The robot can generate a description of the guest and recite these features during the final stage of
the task (T3). The robot can engage in conversation with a newly arrived guest and learn about their name and favourite
drink, it recites these back to the guest as confirmation (T4). The robot is able to learn the guest's face features and
later looks at the correct guest during the final phase (T5). The robot directs guests to the seating area and then
looks towards an empty chair to indicate they should sit in it (T6).

The robot also maintains gaze with the person they talk to during setup by virtue of the guest always being in front of
the robot, and turns to look at the person they intend to talk to during the final phase (N1). Finally, the robot always
looks in the direction of travel except when in simulation due to limitations with the simulated TIAGo (N2).

#figure(
  image("../images/screenshots/Screenshot from 2024-03-14 16-02-19.png", width: 70%),
  caption: [Robot correctly reidentifies a guest and logs their position],
)

=== Assessing face reidentification
#let face_reidentification = json("../data/results2-edited.json");

From the collected experimental data, the mode of the face detection confidence is 99.9%, with lows of 99.7%. However,
we find that in some circumstances it is challenging to detect faces when the image quality is poor as seen with subject
C.

#figure(caption: "Face Detection Results", scale(x: 85%, y: 85%, reflow: true, table(
  columns: (auto, auto, auto),
  stroke: 0.4pt,
  table.header([*Subject*], [*Total*], [*Detected*]),
  "A",
  [15],
  [100%],
  "B",
  [15],
  [100%],
  "C",
  [15],
  [53%],
  "D",
  [15],
  [93%],
  "E",
  [15],
  [100%],
)))

In general, the success rate of reidentification is almost always 100%. In the context of the task, we can confidently
say that at least 5 images will be collected on average which should certainly guarantee that we reliably reidentify
guests.

#figure(
  caption: "Face Reidentification Results",
  table(
    columns: (auto, auto, auto, auto, auto, auto, auto, auto, auto, auto),
    stroke: 0.4pt,
    table.header(
      table.cell(colspan: 1, stroke: (x: none, y: none), []),
      table.cell(colspan: 3, [*Success Rate*]),
      table.cell(colspan: 3, [*Min. Confidence*]),
      table.cell(colspan: 3, [*Avg. Confidence*]),
    ),
    [*Subject*],
    [1:14],
    [2:13],
    [5:10],
    [1:14],
    [2:13],
    [5:10],
    [1:14],
    [2:13],
    [5:10],
    ..for (idx, subject) in (("A", "joe biden"), ("B", "donald trump"), ("B", "sibil"), ("C", "hugh laurie"), ("D", "harry potter"),) {
      ([#idx], ..for ratio in ("1", "2", "5") {
        ([#face_reidentification.reidentify_success_rate.at(ratio).at(subject)],)
      }, ..for ratio in ("1", "2", "5") {
        ([#face_reidentification.reidentify_confidence.at(ratio).at(subject).min],)
      }, ..for ratio in ("1", "2", "5") {
        ([#face_reidentification.reidentify_confidence.at(ratio).at(subject).mean],)
      })
    },
  ),
)

=== Assessing speech recognition
#let speech_system_results = json("../data/results3.json");

In general, the results show that the chance of failure isn't astoundingly high, and given that we stick to the
medium-sized model, we can assume that we are unlikely to mishear someone.

#figure(caption: "Transcription Success Rate", table(
  columns: (auto, auto, auto, auto, auto, auto, auto, auto),
  stroke: 0.4pt,
  table.header(
    table.cell(colspan: 2, stroke: (x: none, y: none), []),
    table.cell(colspan: 3, [*Clean Audio*]),
    table.cell(colspan: 3, [*Busy Coffee Shop*]),
  ),
  [*Phrase*],
  [*Samples*],
  [tiny],
  [small],
  [medium],
  [tiny],
  [small],
  [medium],
  ..for phrase in speech_system_results.transcribe_success_rate.at("tiny.en").at("clean").keys() {
    (["#phrase"], [5], ..for model in ("tiny.en", "small.en", "medium.en") {
      ([#speech_system_results.transcribe_success_rate.at(model).at("clean").at(phrase)],)
    }, ..for model in ("tiny.en", "small.en", "medium.en") {
      ([#speech_system_results.transcribe_success_rate.at(model).at("busy coffee shop").at(phrase)],)
    })
  },
  ..for phrase in speech_system_results.transcribe_success_rate.at("tiny.en").at("untrained phrases").keys() {
    (["#phrase"], [3], ..for model in ("tiny.en", "small.en", "medium.en") {
      ([#speech_system_results.transcribe_success_rate.at(model).at("untrained phrases").at(phrase)],)
    }, table.cell(colspan: 3, stroke: (x: none, y: none), []))
  },
  [*Total*],
  [112],
  table.cell(colspan: 6, stroke: (x: none, y: none), []),
))

The tiny-sized model was surprisingly good, but it often hallucinated phrases or was just completed wrong. For example,
it could interpret "It's finn" as "it's been" or "it's fun", "cola" as "color", "lemonade" as "laminate", and so on. The
worst hallucinations seen were "tropical juice" as "drop good juice", "trump goodness", and "trump could just".

All of these issues are resolved in the larger models. However, the medium-sized model still struggled with recognising
the name "Finn", with this issue being less prevalent in medium. The only outlier in medium was interpreting "charlie"
as "chani".

In terms of then recognising the intent, the assistant was always able to recognise what we meant when the phrases
generated by the transcription were correct. And even then, it was more often right in picking the correct intent and
picking out the information.

#figure(
  caption: "RASA Intent Recognition Results",
  table(
    columns: (auto, auto, auto, auto, auto, auto),
    stroke: 0.4pt,
    table.header(table.cell(colspan: 2, stroke: (x: none, y: none), []), table.cell(colspan: 4, "Correct Inferences")),
    table.cell(colspan: 2, stroke: (x: none, y: none), []),
    table.cell(colspan: 2, "Valid Phrases"),
    table.cell(colspan: 2, "Invalid Phrases"),
    [*Total Phrases*],
    [*Valid Phrases*],
    [*Intents*],
    [*Entities*],
    [*Intents*],
    [*Entities*],
    [#speech_system_results.rasa_results.keys().len()],
    [38],
    [100%],
    [100%],
    [70%],
    [60%],
  ),
)

=== Assessing guest feature extraction

This was the least reliable component of the project as seen in the results below.

#figure(caption: "Person Description Results", table(
  columns: (auto, auto, auto, auto),
  stroke: 0.4pt,
  table.header(table.cell(colspan: 4, "Correct Inferences")),
  [*Torso Colour*],
  [*Hair Colour*],
  [*Glasses Presence*],
  [*Hat Presence*],
  [53%],
  [60%],
  [80%],
  [90%],
))

This can be put down to a combination of factors involving poor image quality (causing colours to tend to extremes),
poor colour estimation (the code took the mean RGB value and compared it to the closest known value, this is relatively
primitive and likely why colours are often wrong), and unrefined feature segmentation model. This will not be considered
to satisfy its respective requirement due to the poor overall performance.

#pagebreak(weak: true)
== Summary

All of the raw data used to assess the experiments is available in Appendix @experiment-test-data.

#figure(caption: "List of project requirements", table(
  columns: (auto, auto),
  align: left,
  stroke: 0.4pt,
  table.header([*Requirement*], [*Satisfied*]),
  [*T1* Able to navigate to preconfigured beacons in arena],
  [R,S],
  [*T2* Wait for a person to appear and approach them],
  [R,S],
  [*T3* Generate a description of a guest including at least 4 characteristics],
  [R,S],
  [*T4* Engage in conversation with a guest asking for name \& favourite drink],
  [R,S],
  [*T5* Learn a guest's face features and then later reidentify them],
  [R,S],
  [*T6* Direct guests to sit in empty seats in the waiting area],
  [R,S],
  [*N1* Robot should maintain gaze with any person it talks to],
  [R,S],
  [*N2* Robot should look in the direction of travel when moving around],
  [R],
  [*N3* Robot can reliably detect and reidentify faces],
  [E],
  [*N4* Robot can reliably recognise speech and intent],
  [E],
  [*N5* Robot can reliably generate the selected characteristics for a guest],
  [],
))

Requirements labeled with *R*, *S*, *E* are satisfied by evidence of the behaviour being observed on the actual robot,
simulation, or in experimentation respectively.