Must be using *Ubuntu 20.04.6 LTS* for proper compatibility with ROS software packages and hence TIAGo robot.

Install prerequisites by either:

- Using #link(
    "https://github.com/LASR-at-Home/Containers/blob/5de81bc0ce5f91cec6e550a553f5de3bab7b272b/robocup_container.def",
  )[#text(blue)[this revision of the RoboCup container]], which you can build using #link("https://lasr-at-home.github.io/docs/guides/container#building")[#text(blue)[these instructions]].
- Install #link(
    "http://wiki.ros.org/Robots/TIAGo/Tutorials/Installation/InstallUbuntuAndROS",
  )[#text(blue)[ROS noetic with TIAGo natively]]
  then install requirements from container definition manually.

Now create your ROS workspace:

```bash
mkdir -p ~/lasr_ws/src
cd ~/lasr_ws/src
git clone -b insert/soft-fork/bsc-receptionist --depth=1 https://github.com/insertish/lasr-base lasr-base
git clone https://git.is.horse/insert/university/undergraduate-project bsc-project

cd ~/lasr_ws
catkin build bsc_receptionist # this may take a while for the first time (10-20 minutes)

# now also download models:
# src/lasr-base/common/vision/lasr_vision_torch/models/best_model_epoch_31.pth
# (find them here: https://tank.insert.moe/archive/projects/bsc-project)

# to run the task later, you must always:
source ~/lasr_ws/devel/setup.bash
```

Train the conversation assistant neural network:

```bash
roscd bsc_receptionist/assistant
rosrun lasr_rasa rasa train nlu
```

Check microphone information:

```bash
rosrun lasr_speech_recognition_whisper list_microphones.py
# Take note of the index corresponding to your desired microphone
```

Start up everything we'll need to run the task:

```bash
roslaunch bsc_receptionist services.launch microphone:=1 robot:=true
# Wait for "All services are ready!" to appear

# You may need to set sample rate:
roslaunch bsc_receptionist services.launch microphone:=1 sample_rate:=44100 robot:=true
# .. individually try running service and see if it crashes:
rosrun lasr_speech_recognition_whisper transcribe_microphone_server --mic_device 1 --sample_rate 44100
```

If you are running for the first time, you must configure the arena:

```bash
rosrun bsc_receptionist configure points set home
rosrun bsc_receptionist configure points set wait_for_person
rosrun bsc_receptionist configure points set seating_area
rosrun bsc_receptionist configure polygons set entrance 4
```

You may also optionally learn the host's name and face data to improve reliability:

```bash
rosrun bsc_receptionist configure learn-host <their_name>
```

Then save the configuration data:

```bash
rosrun bsc_receptionist configure save <name>
```

For subsequent runs, you can either re-select a configuration using the CLI above or run:

```bash
rosrun bsc_receptionist configure load <name>
```

Now you can run the task:

```bash
rosrun bsc_receptionist task 2>&1 | tee "$(date +"%Y_%m_%d_%I_%M_%p").log"

# .. if you want to log what happens:
./record-debug.sh
```

You can choose to run on a simulated robot too:

```bash
roslaunch tiago_2dnav_gazebo tiago_navigation.launch public_sim:=true
rosservice call /gazebo/pause_physics "{}" # save power
roslaunch bsc_receptionist services.launch

# rosservice call /gazebo/unpause_physics "{}" .. automatically called

rosrun bsc_receptionist configure load simulation
rosrun bsc_receptionist task yes 2>&1 | tee "$(date +"%Y_%m_%d_%I_%M_%p").log" # (runs with simulation helpers)

# if you want to see speech output:
rostopic echo /bsc_receptionist/tts
```

#pagebreak()
== Operator Guidance

- Ensure the correct operating procedure is followed for the robot in use.

  Understand the safety features of the robot you are running the task on, make sure you know how to stop the robot in an
  emergency, and always have someone attending the robot.

- Ensure premises have notice that facial recognition is in use.

- Provide users with a notice that data is being collected to perform the task which also states how long, and their
  rights to this data.

- Make sure to clean up any user data that may remain.

  If the task crashes out, face vectors may remain on disk so these must be deleted manually.

- Not save any facial feature vectors to configuration files.

  Or create a process to ensure that this information is wiped after the execution of the task.

// == Privacy Policy

// During the operation of this task, care is taken to collect and store the minimum amount of data required to provide an
// adequately feature-rich service.

// The following data is collected during the execution of the task:

// \begin{itemize} \item \textbf{Face Recognition Vectors}: These are collected during the setup phase when the host is
// onboarded, the guest introduction phase where the robot learns about the guests, and during the guest interaction phase
// where the guest reidentifies guests. This data is kept until the robot is reset, the task ends, and momentarily during
// processing respectively. \item \textbf{Textual Description of Individual}: This is generated during the guest
// introduction phase for the first guest only. This information is kept until the task ends. \item \textbf{Name and
// Favourite Drink}: This information is collected during the setup phase when the host is onboarded and when guests are
// introduced. This data is kept until the robot is reset and when the task ends respectively. \end{itemize}

// Your data is never shared with third parties and is only handled by the operator of the task.

// \subsubsubsection{Your rights under the GDPR}

// The contact information of the data controller of your data will be provided to you by the operator of the task.

// The legal basis for processing your data is based on your consent, which you give us when you acknowledge these terms
// and participate in the task. You can withdraw your consent at any time by leaving the premises and notifying the
// operator.

// As a user of our service, you have the right to complain under the UK General Data Protection Regulation (UK GDPR) if
// you think that we have mishandled your data or violated your data protection rights. You can do this by contacting us
// directly and explaining your concerns. If you are not satisfied with our response, you can also escalate your complaint
// to the Information Commissioner's Office (ICO), which is the supervisory authority for UK GDPR. You can find more
// information about how to make a data protection complaint on their website.

// You may request a copy of your data from the operator.

// \begin{admonition}{Disclosure} The text above contains legal boilerplate adapted from another one of my projects
// \cite{revolt_privacy}. \end{admonition}