#import "lib/admonitions.typ": todo
#import "@preview/lovelace:0.2.0": *
#import "lib/modern-unito-thesis.typ": template
#import "@preview/colorful-boxes:1.1.0": stickybox
#import "@preview/wordometer:0.1.0": total-words, word-count
#import "@preview/drafting:0.2.0": set-page-properties, set-margin-note-defaults

// change to true for final export
#let production = "PRODUCTION" in sys.inputs;

#set document(title: "Robot Receptionist", author: "Pawel Makles")
#set page(paper: "a4", margin: (right: 3cm, left: 3.5cm, top: 3.5cm, bottom: 3.5cm), footer: context[
  #if production [
    #text(gray)[6CCS3PRJ Report Final Copy]
  ] else [
    #text(red)[*DRAFT COPY #datetime.today().display()*]
  ]
  #h(1fr)
  #counter(page).display("1/1", both: true)
])

// configure lovelace
#show: setup-lovelace
// configure drafting:
#set-page-properties()
// configure word count:
#show: word-count.with(exclude: strike)

// configure codly
#import "@preview/codly:0.2.0": *
#show: codly-init.with()

#let icon(codepoint) = {
  box(height: 0.8em, baseline: 0.05em, image(codepoint))
  h(0.1em)
}

#codly(languages: (
  rust: (name: "Rust", icon: icon("images/icons/brand-rust.svg"), color: rgb("#CE412B")),
  yaml: (name: "YAML", icon: icon("images/icons/brand-yaml.svg"), color: rgb("#35A235")),
  python: (name: "Python", icon: icon("images/icons/brand-python.svg"), color: rgb("#3572A5")),
), zebra-color: white)

#show figure.where(kind: table): set figure.caption(position: top)
#show figure.where(kind: "data"): set figure.caption(position: top)

// just some fun, enable sticky notes:
#let default-rect(stroke: none, fill: none, width: 0pt, content) = {
  set text(0.9em)
  stickybox(rotation: 20deg, width: width, content)
}
#set-margin-note-defaults(rect: default-rect, stroke: rgb(0, 0, 0, 20), side: right)

#let declaration-of-originality = [
  I verify that I am the sole author of this report, except where explicitly stated to the contrary. I grant the right to
  King's College London to make paper and electronic copies of the submitted work for purposes of marking, plagiarism
  detection, and archival, and to upload a copy of the work to Turnitin or another trusted plagiarism detection service. I
  confirm this report does not exceed 25,000 words.

  #set align(right)
  Pawel Makles

  April 12, 2024

  Word Count: #total-words
]

#let acknowledgments = [
  I would like to express my gratitude to Dr. Matteo Leonetti for the continued help and support throughout the project. I would also like to extend my thanks to Jared Swift, Matt Barker, and Benteng Ma for assisting in testing my project in the lab and for their contributions to the LASR code repository which proved to be a vital foundation for the development of this project.
]

#let abstract = [

  Service robotics is a rapidly growing field with many applications from hospitality to medicine creating real-world impact on real people in the service industry. This project intends to explore the application of service robots to a social setting, specifically a gathering of people where the robot acts as the receptionist, greeting new guests, and introducing guests.

  This project has successfully reached its intended goal and can successfully perform the social interactions that are asked of it. However, some areas of improvement have been identified which are discussed in the evaluation.
]

#show: template.with(
  title: "Robot Receptionist",
  academic-year: [2023/2024],
  subtitle: "6CCS3PRJ Project Report",
  candidate: (name: "Pawel Makles", number: "K21002534"),
  supervisor: ("Dr. Matteo Leonetti"),
  co-supervisor: none,
  affiliation: (university: "King's College London", school: "Department of Informatics", degree: "MSci Computer Science"),
  lang: "en",
  logo: image("images/template/logo.png", width: 40%),
  bibliography: bibliography("references.bib", style: "ieee"),
  declaration-of-originality: declaration-of-originality,
  acknowledgments: acknowledgments,
  abstract: abstract,
  keywords: none,
)

#let section(title, body, finished: false) = [
  #word-count(total => [
    #pagebreak(weak: true)
    #if production [
      = #title
    ] else [
      = #title (#total.words words, #if finished [#text(rgb(0, 180, 0))[finished]] else [#text(red)[incomplete]])
    ]

    #body
  ])
]

#section("Introduction", include "chapters/1-introduction.typ", finished: true)
#section("Background & Context", include "chapters/2-background.typ", finished: true)
#section("Literature", include "chapters/3-literature.typ", finished: true)
#section("Specification & Design", include "chapters/4-specification-design.typ", finished: true)
#section("Implementation", include "chapters/5-implementation.typ", finished: true)
#section("Evaluation", include "chapters/6-evaluation.typ", finished: true)
#section("Legal, Social, Ethical, and Professional Issues", include "chapters/7-issues.typ", finished: true)
#section("Conclusion", include "chapters/8-conclusions.typ", finished: true)

// Create a new project:
#pagebreak(weak: true)
#set heading(numbering: "A.1.1")
#counter(heading).update(0)
#hide[
  = User Manual <user-manual>
  = Receptionist Task Rules <robocup-task-rules>
  = Raw Experiment Test Data <experiment-test-data>
]

Appendix omitted, provided in separate PDF.

Keep scrolling for references.
