#import "@preview/polylux:0.3.1": *
#import "lib/diagrams.typ": fletcher, edge, node, ros-node, blob

#import themes.simple: *

#set text(font: "Inria Sans")

#show: simple-theme.with(footer: [6CCS3PRJ])

#title-slide[
  #grid(columns: (auto, auto), gutter: 128pt, [
    6CCS3PRJ

    = Robot Receptionist
    #v(2em)

    Pawel Makles#linebreak()
    #text(size: 14pt)[K21002534]

  ], image("images/screenshots/vlcsnap-2024-04-18-13h46m04s664.png", height: 100%))
]

#slide[
  = Introduction & Goals

  - Why?
    #pause
    - Service robotics is a growing field that we want to explore
    #pause
    - Help for the home, hospitality, telepresence, logistics, ...
  #pause
  - What?
    #pause
    - RoboCup\@Home is a major service robotics competition
    #pause
    - Robot Receptionist

]

centered-slide[= Implementation ]

#slide[
  #grid(columns: (auto, auto), gutter: 40pt, [
    == Services

    First, we need:
    #line-by-line[
      - Understanding of TIAGo
      - Object Detection #text(size: 10pt)[(YOLO)]
      - Body Segmentation #text(size: 10pt)[(BodyPix)]
      - Face Feature Extraction #text(size: 10pt)[(Torch)]
      - Face Recognition #text(size: 10pt)[(DeepFace)]
      - Speech Recognition #text(size: 10pt)[(Whisper)]
      - Dialog Intent #text(size: 10pt)[(RASA)]
    ]
  ], [
    #uncover(1, place(horizon + left, box([
      - Navigation Stack
      - Depth Information
      - Text-to-speech
      - RGB Camera
    ])))

    #uncover(2, place(horizon + left, image("images/inference/yolo.png")))
    #uncover(3, place(horizon + left, image("images/inference/bodypix.png")))
    #uncover(4, place(horizon + left, image("images/inference/torch_out1.png")))
    #uncover(5, place(horizon + left, image("images/inference/deepface.png", width: 120pt)))

    #uncover(8, mode: "invisible")[
      #let tiagoTopicMoveBase = -0.22;
      #let tiagoTopicCamera = 0.28;
      #let tiagoTopicCameraI = 0.03;
      #scale(x: 50%, y: 50%, reflow: true, fletcher.diagram(
        node-stroke: 0.5pt,
        edge-stroke: 0.7pt,
        ros-node((0, 0), "Task", tint: green),
        ros-node((1, 1), "Object Detection", [*service* /detect], tint: purple),
        ros-node((1, 2), "Body Segmentation", [*service* /segment], [*service* /face_features], tint: purple),
        ros-node((1, 3), "RASA", [*service* /parse], tint: purple),
        ros-node((2, 3), "Whisper", [*action* /transcribe], tint: purple),
        ros-node((2, 2), "DeepFace", [*service* /vectorise], tint: purple),
        ros-node((2, 1), "Weaviate", [*service* /insert], [*service* /query], tint: red),
        ros-node(
          (2, 0),
          "TIAGo",
          [*action* /move_base],
          [*topic* /pointcloud],
          [*action* /tts],
          [*topic* /camera/rgb_raw],
          tint: red,
        ),
        // task => nodes
        edge((0, 0), (2, tiagoTopicMoveBase), bend: 3deg),
        edge((0, 0), (2, tiagoTopicMoveBase + 0.24), bend: 3deg),
        edge((0, 0), (2, tiagoTopicMoveBase + 0.44), bend: 3deg),
        edge((0, 0), (0, 1.1), (1, 1.1)),
        edge((0, 0), (0, 2.12), (1, 2.12)),
        edge((0, 0), (0, 3.15), (1, 3.15)),
        // nodes => tiago
        edge((1, 1.1), (1.4, 1.1), (1.4, tiagoTopicCamera), (2, tiagoTopicCamera), "--"),
        edge(
          (1, 1.98),
          (1.42, 1.98),
          (1.42, tiagoTopicCamera + tiagoTopicCameraI),
          (2, tiagoTopicCamera + tiagoTopicCameraI),
          "--",
          "{img}",
        ),
        edge(
          (1, 2.2),
          (1.44, 2.2),
          (1.44, tiagoTopicCamera + tiagoTopicCameraI * 2),
          (2, tiagoTopicCamera + tiagoTopicCameraI * 2),
          "--",
        ),
        // body => deepface
        edge((1, 2.02), (1.5, 2.02), (1.5, 2.1), (2, 2.1), "-->"),
        // rasa => whisper
        edge((1, 3.15), (2, 3.15), "<--", "{speech}"),
        // deepface => weaviate
        edge((2, 2.1), (2.65, 2.1), (2.65, 1.2), (2, 1.2), "--"),
        edge((2, 2.14), (2.68, 2.14), (2.68, 1), (2, 1), "--", "{vec}"),
      ))
    ]
  ])
]

#slide[
  == Task

  #grid(columns: (auto, auto), [
    #line-by-line[
      - Implemented using SMACH
      - Overarching state machine
        - Meet the guest
        - Seat the guest
        - Introduce guests
    ]
  ], [
    #uncover(2, place(top + left, scale(x: 40%, y: 40%, reflow: true, fletcher.diagram(
      node-stroke: 0.5pt,
      edge-stroke: 0.7pt,
      blob((0, 0), "Start", tint: gray),
      edge("-->", "(sim)"),
      blob((1, 0), "Setup Sim"),
      edge("->"),
      blob((0, 1), "Enable Head"),
      edge((0, 0), (0, 1), "->"),
      edge("->"),
      blob((0, 2), "Navigate to start"),
      edge("->"),
      blob((-1, 2), "Speak \"Starting task\""),
      edge("->"),
      blob((-1, 3), "Run for two guests", tint: yellow),
      edge("->", "i=1..2"),
      blob((0, 3), "Navigate to waiting area"),
      edge("->"),
      blob((1, 3), "Reset Head"),
      edge("->"),
      blob((1, 4), "Wait for a person", tint: red),
      edge("->"),
      blob((0, 4), "Meeting the guest", tint: red),
      edge("->"),
      blob((0, 5), "Enable Head"),
      edge("->"),
      blob((0, 6), "Seating guest", tint: red),
      edge((0, 6), (-0.5, 6), (-1, 3), "->", bend: 100deg),
      blob((-1, 6), "Navigate to seats"),
      edge((-1, 3), (-1, 6), "-->", "i>2"),
      edge("->"),
      blob((-1, 7), "Introducing guests", tint: red),
      edge("->"),
      blob((0, 7), "Navigate to end"),
      edge("->"),
      blob((1, 7), "Speak ``Finished task''", tint: gray),
    ))))
  ])
]

#centered-slide[
  = Evaluation
]

#slide[
  == Does it work?
  #pause
  Yes.

  #uncover(2, box(place(top + left, [
    #set enum(numbering: "T1")

    + #emoji.checkmark.heavy Able to navigate to preconfigured beacons in the arena
    + #emoji.checkmark.heavy Wait for a person to appear and approach them
    + #emoji.checkmark.heavy Generate a description of a guest incl. at least 4 characteristics
    + #emoji.checkmark.heavy Engage in conversation with a guest asking for name \& drink
    + #emoji.checkmark.heavy Learn a guest's face features and then later reidentify them
    + #emoji.checkmark.heavy Direct guests to sit in empty seats in the waiting area
  ])))

  #pause
  #v(2em)
  == Does it work well?
  #pause
  For the most part. Falling short in feature extraction and some robustness. Limitations of hardware.
]

#centered-slide[
  = Lab Demo
]

#centered-slide[
  = Simulation Demo
]

#slide[
  = Conclusion

  #pause
  - Mostly met criteria, missed the mark in feature extraction
  #pause
  - State-of-the-art models are the path of least resistance
  #pause
  - Development of custom vision models is a project in itself
  #pause
  - SMACH is an excellent library (for prototypes & production)
]

#centered-slide[
  #grid(
    row-gutter: 16pt,
    [Presentation made with],
    image("images/icons/kdenlive.png", height: 1.5em),
    image("images/icons/typst.png", height: 2em),
    image("images/icons/polylux.png", height: 2em),
    v(2em),
    [Source code available at],
    text(size: 16pt, link(
      "https://git.is.horse/insert/university/ug-project-dissertation/-/blob/main/README.md",
      "git.is.horse/insert/university/ug-project-disseratation",
    )),
  )
]
